/*
*   Class  Strings
*
*   Methods for editing Strings

*   WRITTEN BY: Dr Michael Thomas Flanagan
*
*   DATE:       March 2013
*   REVISED:    April 2013, 2 May 2013, 30 November 2013, 20 July 2015, 15 June to 21 July2020, 10-16 August 2020
*
*   DOCUMENTATION:
*   See Michael Thomas Flanagan's Java library on-line web page:
*   http://www.ee.ucl.ac.uk/~mflanaga/java/Strings.html
*   http://www.ee.ucl.ac.uk/~mflanaga/java/
*
*   Copyright (c)2013 Michael Thomas Flanagan
*
*
***************************************************************************************/
package flanagan.util;

import java.util.ArrayList;

import flanagan.math.ArrayMaths;
import flanagan.math.DeepCopy;
import flanagan.math.Fmath;
import flanagan.io.Db;
import flanagan.io.DialogueBox;
import flanagan.io.FileInput;
import flanagan.io.FileManipulate;
import flanagan.io.FileOutput;
import flanagan.io.PrintToScreen;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

public class Strings{

    private String enteredString = null;                        // Entered string
    private String originalString = null;                       // Original entered String
    private String editedString = null;                         // Edited string
    private String asciiString = null;                          // all codes converted toASCII char i the entered string
    private int nEntered = 0;                                   // Number of characters in the entered string after any html to ASCII conversions
    private int nOriginal = 0;                                  // Number of characters in the original entered string
    private int nAsciiString = 0;                               // Number of characters in the ASCII string
    private String upperCaseString = null;                      // Upper case string
    private String lowerCaseString = null;                      // Lower case string
    private String allTitleCaseString = null;                   // All title string
    private String titleCaseString = null;                      // Title string
    private String sentenceCaseString = null;                   // Sentence case string
    private String unAccentedString = null;                     // Unaccented string
    private int[] asciiDecimalString = null;                    // ASCII decmal equivalents of each character in the Strig
    private int[] whiteSpaceIndices = null;                     // indices of the white space
    private boolean[] whiteSpaces = null;                       // true if corresponding char is a white space
    private int nWhiteSpaces = 0;                               // number of white spaces
    private String[] tokens = null;                             // tokens separated by white space/s
    private int nTokens = 0;                                    // number of tokens
    private String separator = null;                            // tokens separatop
    private int[] tokenInitialIndices = null;                   // initial index of each token
    private int[] tokenFinalIndices = null;                     // final index of each token
    private boolean tokensDone = false;                         // = true when tokens found
    private String[] principalTokens = null;                    // tokens, other tan the first token, that are not articles, prepostions or conjunctions
    private int nPrincipalTokens = 0;                           // number of principal tokens
    private int[] principalTokenInitialIndices = null;          // initial index of each principal token
    private int[] principalTokenFinalIndices = null;            // final index of each principaltoken
    private boolean suppressDialogue = false;                   // Suppress dialogue indicator
    private boolean tabastab = false;                           //  if false tab replaced by white space
    private boolean tabsReplaced = false;                       // = true if tabs replaced by spaces
    private int[] singleQuotationMarks = null;                  // indices of single quotation marksn
    private int[] singleQuotationMarksDec = null;               // ASCII decimal equivalents of single quotation marks
    private int nSingleQuotationMarks = 0;                      // number of single quotation marks
    private boolean singleQuotationMarksDone = false;           // = true when single quotation marks indices and ASCII calculated
    private int[] doubleQuotationMarks = null;                  // indices of double quotation marksn
    private int[] doubleQuotationMarksDec = null;               // ASCII decimal equivalents of double quotation marks
    private int nDoubleQuotationMarks = 0;                      // number of double quotation marks
    private boolean doubleQuotationMarksDone = false;           // = true when double quotation marks indices and ASCII calculated
    private int[] quotationMarks = null;                        // indices of single and double quotation marksn
    private int[] quotationMarksDec = null;                     // ASCII decimal equivalents of single and double quotation marks
    private int nQuotationMarks = 0;                            // number of single and double quotation marks
    private boolean quotationMarksDone = false;                 // = true when sngle and double quotation marks indices and ASCII calculated
    private int[] possessiveApostrophes = null;                 // indices of any possessive apostrophes
    private int[] possessiveApostropheDecimals = null;          // ASCII decimal equivalents of any possessive apostrophes  
    private int nPossessiveApostrophes = 0;                     // number of possessive apostrophes
    private boolean possessiveApostrophesDone = false;          // = true when possessive apostropes found in the entered string
    private int[] surnameApostrophes = null;                   // indices of any surname apostrophes
    private int[] surnameApostropheDecimals = null;             // ASCII decimal equivalents of any surname apostrophes  
    private int nSurnameApostrophes = 0;                        // number of surname apostrophes
    private boolean surnameApostrophesDone = false;             // = true when surname apostropes found in the entered string
    private int[] dashes = null;                                // indices of any dashes
    private int[] dashesDec = null;                             // ASCII decimal equivalents of any dashes  
    private int nDashes = 0;                                    // number of dashes
    private boolean dashesDone = false;                         // = true when sdashes found in the entered string
    private int htmlOption = 1;                                 // = 1    ASCII and HTML option = Windows-1252 (CP-1252) in decimal range 128 to 156
                                                                // = 2  aASCII and HTML option = ISO-8859-1 in decimal range 128 to 156
                                                                // = 3  aASCII and HTML option = control set C1 in decimal range 128 to 156
    private String asciiHtmlArrayFilename = "htmlArraysWin.txt";// File name for entered ascii and html arrays                                 
    
    // Accent  arrays
    private String[] symbolsA = null;
    private String[] replSymbolsA = null;
    private String[] htmlNamesA = null;
    private String[] htmlNumbersA = null;
    private int[] decNumbersA = null;
    private int nSymbolsA = -1;

    // Quotation mark  arrays   
    private String[] symbolQ = null;
    private String[] htmlNamesQ = null;
    private String[] htmlNumbersQ = null;
    private int[] decNumbersQ = null;
    private int nSymbolsQ = -1;

    // Single quotation mark arrays   
    private String[] symbolSQ = null;
    private String[] htmlNamesSQ = null;
    private String[] htmlNumbersSQ = null;
    private int[] decNumbersSQ = null;
     private int nSymbolsSQ = -1;

    //Double quotation mark arrays   
    private String[] symbolDQ = null;
    private String[] htmlNamesDQ = null;
    private String[] htmlNumbersDQ = null;
    private int[] decNumbersDQ = null;
    private int nSymbolsDQ = -1;

    // Apostrophes  
    private String[] symbolsAp = null;
    private String[] htmlNamesAp = null;
    private String[] htmlNumbersAp = null;
    private int[] decNumbersAp = null;
    private int nSymbolsAp = -1;

    // dash arrays
    private String[] symbolsD = null;
    private String[] htmlNamesD = null;
    private String[] htmlNumbersD = null;
    private int[] decNumbersD = null;
    private int nSymbolsD = -1;

    // All codes listed
    private String[] htmlNumbersAll = null;
    private String[] htmlNamesAll = null;
    private int[] multipleNamesAll = null;
    private String[] commentsAll = null;
    private int[] decNumbersAll = null;
    private String[] hexadecsAll = null;
    private String[] htmlHexadecimalsAll = null;
    private int[] multipleCodesAll = null;
    private String[] typesAll = null;
    private String[] repeatsAll = null;
    private int nSymbolsAll = -1;
    private int[] controlCharIndicesAll = null;
    private String[] controlCharSymbolsAll = null;
    private int nControlsAll = -1;
  
    // all name codes listed
    public String[] htmlNamesNA = null;
    public int[] decNumbersNA = null;            
    public String[] hexadecsNA = null;
    public String[] htmlNumbersNA = null;
    public String[] htmlHexadecimalsNA = null;
    public String[] commentsNA = null;
    public int[] multipleNamesNA = null;
    public int[] multipleCodesNA = null;
    public int nSymbolsNA = -1;

    // Metacharacters
    private char[] metacharactersC = {'\\', '^', '$', '.', '|', '?', '*', '+', '(', ')', '[', '{'};
    private String[] metacharactersS = {"\\", "^", "$", ".", "|", "?", "*", "+", "(", ")", "[", "{"};
    private int nMeta = metacharactersS.length;

    // Articles
    private String[] articles = {"a", "an", "the"};
    private int nArticles = articles.length;
    
// Prepositions
    private String[] prepositions = {"about", "above", "across", "after", "against", "along", "among", "around", "at", "before", "behind", "below", "beneath", "beside", "between", "beyond", "but", "by", "despite", "down", "during", "except", "for", "from", "in", "inside", "into", "like", "near", "of", "off", "on", "onto", "out", "outside", "over", "past", "since", "through", "throughout", "till", "to", "toward", "under", "underneath", "until", "up", "upon", "with", "within", "without"};
    private int nPrepositions = prepositions.length;
    // Conjunctions
    private String[] conjunctions = {"and", "&amp", "&", "but", "or", "nor", "for", "so", "yet", "not", "only", "also", "either", "neither", "although", "because", "since", "unless", "until", "while"};
    private int nConjunctions = conjunctions.length;

    // Constructors 
    public Strings(String string){
        this.enteredString = string;
        try{
            this.readArrays();
        }catch(java.io.IOException e){
            System.out.println("Constructor error 1 " + e);
        }
        this.setData(string);
    }


    public Strings(){
        try{
             this.readArrays();
        }catch(java.io.IOException e){
             System.out.println("Constructor error 2 " + e);
        }
    }

    // Set entered data
    private void setData(String string) {
        this.originalString = string;
        this.nOriginal = string.length();
        this.enteredString = string;
        this.nEntered = this.enteredString.length();    
        //this.asciiString =  this.allHtmlNumberCodesToAsciiChars();
        //this.asciiString =  this.allHtmlNameCodesToAsciiChars();
        //this.nAsciiString = this.asciiString.length();
    }
    
    // Set encoding tabulation
    public void setEncodingTabulation(int option){
        switch(option){
            case 1: this.asciiHtmlArrayFilename = "htmlArraysWin.txt";
                    this.htmlOption = 1;
                    break;
            case 2: this.asciiHtmlArrayFilename = "htmlArraysIso.txt";
                    this.htmlOption = 2;
                    break;
            case 3: this.asciiHtmlArrayFilename = "htmlArraysC1.txt";
                    this.htmlOption = 3;
                    break;
        }
    }
    
    // Read html data arrays
    public void readArrays() throws IOException{
        String line = null;
        String hold0 = null;
        String[] tokens = null;
        ArrayList<String> al0 = new ArrayList<String>();
        int pos0 = 0;
        int ii = 0;
        try{
            InputStream is = getClass().getResourceAsStream(this.asciiHtmlArrayFilename);
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            //line = br.readLine();
            boolean testr = true;
            while (testr){
                line = br.readLine();
                while(pos0>-1){
                    pos0 = line.indexOf(',');
                    if(pos0>-1){
                        hold0 = line.substring(0,pos0).trim();
                        al0.add(hold0);
                        line = line.substring(pos0+1);  
                    }
                    else{
                        al0.add(line.trim());
                    }
                }
                pos0 = 0;
                int nal = al0.size();
                tokens = new String[nal];
                for(int j=0; j<nal; j++)tokens[j] = al0.get(j);
                al0 = new ArrayList<String>();
                switch(ii){
                    case 0: this.symbolsA = tokens;
                            ii++;
                            break;
                    case 1: this.replSymbolsA = tokens;
                            ii++;
                            break;
                    case 2: this.htmlNamesA = tokens;
                            ii++;
                            break;
                    case 3: this.htmlNumbersA = tokens;
                            ii++;
                            break;
                    case 4: int ll = tokens.length;
                            int[] dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.decNumbersA = dec;
                            ii++;
                            break;
                    case 5: this.symbolQ = tokens;
                            ii++;
                            break;
                    case 6: this.htmlNamesQ = tokens;
                            ii++;
                            break;
                    case 7: this.htmlNumbersQ = tokens;
                            ii++;
                            break;
                    case 8: ll = tokens.length;
                            dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.decNumbersQ = dec;
                            ii++;
                            break;
                    case 9: this.symbolSQ = tokens;
                            ii++;
                            break;
                    case 10: this.htmlNamesSQ = tokens;
                            ii++;
                            break;
                    case 11: this.htmlNumbersSQ = tokens;
                            ii++;
                            break;
                    case 12: ll = tokens.length;
                            dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.decNumbersSQ = dec;
                            ii++;
                            break;
                    case 13: this.symbolDQ = tokens;
                            ii++;
                            break;
                    case 14: this.htmlNamesDQ = tokens;
                            ii++;
                            break;
                    case 15: this.htmlNumbersDQ = tokens;
                            ii++;
                            break;
                    case 16: ll = tokens.length;
                            dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.decNumbersDQ = dec;
                            ii++;
                            break;
                    case 17: this.symbolsAp = tokens;
                            ii++;
                            break;
                    case 18: this.htmlNamesAp = tokens;
                            ii++;
                            break;
                    case 19: this.htmlNumbersAp = tokens;
                            ii++;
                            break;
                    case 20: ll = tokens.length;
                            dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.decNumbersAp = dec;
                            ii++;
                            break;
                    case 21: this.symbolsD = tokens;
                            ii++;
                            break;
                    case 22: this.htmlNamesD = tokens;
                            ii++;
                            break;
                    case 23: this.htmlNumbersD = tokens;
                            ii++;
                            break;
                    case 24: ll = tokens.length;
                            dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.decNumbersD = dec;
                            ii++;
                            break;
                    case 25: this.htmlNumbersAll = tokens;
                            ii++;
                            break;
                    case 26: this.htmlNamesAll = tokens;
                            ii++;
                            break;                        
                    case 27: ll = tokens.length;
                            dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.multipleNamesAll = dec;
                            ii++;
                            break;       
                    case 28: this.commentsAll = tokens;
                            ii++;
                            break;                                               
                    case 29: ll = tokens.length;
                            dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.decNumbersAll = dec;
                            ii++;
                            break;
                    case 30: this.hexadecsAll = tokens;
                            ii++;
                            break;
                    case 31: this.htmlHexadecimalsAll = tokens;
                            ii++;
                            break;
                    case 32: ll = tokens.length;
                            dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.multipleCodesAll = dec;
                            ii++;
                            break;                    
                    case 33: this.typesAll = tokens;
                            ii++;
                            break;
                    case 34: this.repeatsAll = tokens;
                            ii++;
                            break;        
                    case 35: ll = tokens.length;
                            dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.controlCharIndicesAll = dec;
                            ii++;
                            break;          
                    case 36: this.controlCharSymbolsAll = tokens;
                            ii++;
                            break;  
                    case 37: this.htmlNumbersNA = tokens;
                            ii++;
                            break;
                    case 38: this.htmlNamesNA = tokens;
                            ii++;
                            break;                        
                    case 39: ll = tokens.length;
                            dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.multipleNamesNA = dec;
                            ii++;
                            break;       
                    case 40: this.commentsNA = tokens;
                            ii++;
                            break;                                               
                    case 41: ll = tokens.length;
                            dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.decNumbersNA = dec;
                            ii++;
                            break;
                    case 42: this.hexadecsNA = tokens;
                            ii++;
                            break;
                    case 43: this.htmlHexadecimalsNA = tokens;
                            ii++;
                            break;     
                    case 44: ll = tokens.length;
                            dec = new int[ll];
                            for(int j=0; j<ll; j++)dec[j] = Integer.parseInt(tokens[j]);
                            this.multipleCodesNA = dec;
                            ii++;
                            break;
                }
                if(ii==45)testr=false;
            }
            br.close();
            isr.close();
            is.close();
        }catch(java.io.FileNotFoundException e){
            System.out.println("Error "  + e);
        }
        
        this.nSymbolsA = this.symbolsA.length;
        this.nSymbolsQ = this.symbolQ.length;
        this.nSymbolsSQ = this.symbolSQ.length;
        this.nSymbolsDQ = this.symbolDQ.length;
        this.nSymbolsAp = this.symbolsAp.length;
        this.nSymbolsD = this.symbolsD.length;
        this.nSymbolsAll= this.htmlNumbersAll.length;
        this.nSymbolsNA= this.htmlNumbersNA.length;
        this.nControlsAll = this.controlCharSymbolsAll.length;
                        
    }
    
    // Set html option
    private void setHtmlOption(String opt) {
        if(opt.equalsIgnoreCase("win")){
            this.htmlOption = true;
            this.asciiHtmlArrayFilename = "htmlArraysWin.txt";
        }
        else{
            if(opt.equalsIgnoreCase("iso")){
                this.htmlOption = false;
                this.asciiHtmlArrayFilename = "htmlArraysIso.txt";
            }
            else{
                System.out.println("Method setHtmlOption: Entered HTML array optionn, " + opt + ", is not meaningful\n Windows-1252 (CP-1252) used");
            }
        }
    }

    // Suppress dialogue method
    public void suppressDialogue() {
        this.suppressDialogue = true;
    }

    // Enter a String
    public void setEntered(String string) {
        this.setData(string);
    }
    
    // Enter a String from file
    public void fileEntered(String filename) {
        FileManipulate fin = new FileManipulate(filename);
        String string = fin.toSingleLine();
        this.setData(string);
    }

    // Return enteredString 
    public String getEnteredString() {
        return this.enteredString;
    }

    // Return editedString 
    public String getEditedString() {
        return this.editedString;
    }

    // Return number of characters in the entered String 
    public int getEnteredStringLength() {
        return this.nEntered;
    }

    // get all entered data
    // all html decimal codes
    public String[] getAllHtmlDecimalCodes(){
        return this.htmlNumbersAll;        
    }
    
    // all html hexadecimal codes
    public String[] getAllHtmlHexadecimalCodes(){
        return this.htmlHexadecimalsAll;        
    }
    
    // all html named codes
    public String[] getAllHtmlNamedCodes(){
        return this.htmlNamesAll;       
    }
    
    // all ascii decimal codes
    public int[] getAllAsciiDecimalCodes(){
        return this.decNumbersAll;        
    }
    
    // all ascii hexadecimal codes
    public String[] getAllAsciiHexadecimalCodes(){
        return this.hexadecsAll;        
    }
    
    // all comments
    public String[] getAllComments(){
        return this.commentsAll;        
    }
    
    // all multiple names
    public int[] getAllMultipleNumbers(){
        return this.multipleNamesAll;        
    }
    
   // all multiple codes
   public int[] getAllMultipleCodes(){
        return this.multipleCodesAll;        
   }
   
   // all html named codes
    public String[] getAllNamedHtmlNamedCodes(){
        return this.htmlNamesNA;       
    }
    
    // all ascii decimal codes
    public int[] getAllNamedAsciiDecimalCodes(){
        return this.decNumbersNA;        
    }
    
    // all ascii hexadecimal codes
    public String[] getAllNamedAsciiHexadecimalCodes(){
        return this.hexadecsNA;        
    }
    
    // all comments
    public String[] getAllNamedComments(){
        return this.commentsNA;        
    }
    
    // all multiple names
    public int[] getAllNamedMultipleNumbers(){
        return this.multipleNamesNA;        
    }
    
 
   // number of all codes
   public int getNumberOfAllCodes(){
        return this.nSymbolsAll;        
   }
   
      // number of all codes
   public int getNumberOfAllNamedCodes(){
        return this.nSymbolsNA;        
   }
    
    // get indices of a given decimal - all
   public int[] getIndexDecAll(int dec){
       ArrayList<Integer> al0 = new ArrayList<Integer>();
       
       int n1 = this.nSymbolsAll;
       for(int i=0; i<n1; i++){
           if(dec==this.decNumbersAll[i]){
                 al0.add(i);
           }
       }
       int n2 = al0.size();
       int[] retn = new int[n2];
       for(int i=0; i<n2; i++)retn[i] = al0.get(i);
       return retn;
   }
    
    // CONVERSIONS
    // Reverse String
    // Instance method
    public String reverse() {
        String retn = " ";
        for (int i = this.nEntered - 1; i >= 0; i--) {
            retn += this.enteredString.charAt(i);
        }
        return retn;
    }

    // Static method
    public static String reverse(String ss) {
        String retn = " ";
        for (int i = ss.length() - 1; i >= 0; i--) {
            retn += ss.charAt(i);
        }
        return retn.trim();
    }

   
    // converts all the string to upper case
    public String toUpperCase() {
        this.upperCaseString = this.enteredString.toUpperCase();
        return this.upperCaseString;
    }

    public static String toUpperCase(String ss)  {
        Strings sss = new Strings(ss);
        return sss.toUpperCase();
    }

    // converts to the character at i to upper case
    public String toUpperCase(int i) {
        String upperI = this.enteredString.substring(0, i) + this.enteredString.substring(i, i + 1).toUpperCase() + this.enteredString.substring(i + 1);
        return upperI;
    }

    public static String toUpperCase(String ss, int i)  {
        Strings sss = new Strings(ss);
        return sss.toUpperCase(i);
    }

    // converts to lower case
    public String toLowerCase() {
        this.lowerCaseString = this.enteredString.toLowerCase();
        return this.lowerCaseString;
    }

    // converts to the character at i to lower case
    public String toLowerCase(int i) {
        String lowerI = this.enteredString.substring(0, i) + this.enteredString.substring(i, i + 1).toLowerCase() + this.enteredString.substring(i + 1);
        return lowerI;
    }

    public static String toLowerCase(String ss, int i) {
        Strings sss = new Strings(ss);
        return sss.toLowerCase(i);
    }

    // Return string in full title case
    // instance method
    public String toAllTitleCase() {

        this.allTitleCaseString = this.enteredString.toLowerCase();
        this.tokens();

        int jj = 0;
        for (int i = 0; i < this.nTokens; i++) {
            jj = this.tokenInitialIndices[i];
            if (Chars.isQuotationMark(this.enteredString.charAt(jj)) && jj < this.nEntered) {
                jj++;
            }
            if (!this.whiteSpaces[jj]) {
                this.allTitleCaseString = this.toUpperCase(jj);
            }
        }
        return this.allTitleCaseString;
    }

    // Return string in full title case
    // static method
    public static String toAllTitleCase(String ss)  {
        Strings strs = new Strings(ss);
        return strs.toAllTitleCase();
    }

    // Return string in title case, i.e. only prncipal words with initial upper case
    // instance method
    public String toTitleCase() {
        this.titleCaseString = this.enteredString.toLowerCase();
        this.principalTokens();

        int jj = 0;
        for (int i = 0; i < this.nPrincipalTokens; i++) {
            jj = this.principalTokenInitialIndices[i];
            if (Chars.isQuotationMark(this.titleCaseString.charAt(jj)) && jj < this.nEntered) {
                jj++;
            }
            if (!this.whiteSpaces[jj]) {
                this.toUpperCase(jj);
            }
        }

        // check for colon
        String[] tokens = this.tokens(this.titleCaseString);
        int nT = tokens.length;
        String newS = tokens[0];
        boolean colonFound = false;
        for (int i = 0; i < nT - 1; i++) {
            int nTi = tokens[i].length();
            char cti = tokens[i].charAt(nTi - 1);
            if (cti == ':') {
                colonFound = true;
                tokens[i + 1] = tokens[i + 1].substring(0, 1).toUpperCase() + tokens[i + 1].substring(1);
            }
            newS = newS + " " + tokens[i + 1];
        }
        if (colonFound) {
            this.titleCaseString = newS.trim();
        }

        return this.titleCaseString;
    }

    // Return string in title case, i.e. only prncipal words with initial upper case
    // static method
    public static String toTitleCase(String ss)  {
        Strings strs = new Strings(ss);
        return strs.toTitleCase();
    }

    // Converts to sentence case
    // instance method
    public String toSentenceCase() {
        this.sentenceCaseString = this.enteredString.toLowerCase();
        this.tokens();

        int jj = this.tokenInitialIndices[0];
        int kk = -1;
        if (Chars.isQuotationMark(this.sentenceCaseString.charAt(jj)) && jj < this.nEntered) {
            jj++;
        }
        if (!this.whiteSpaces[jj]) {
            this.toUpperCase(jj);
        }
        for (int i = 0; i < this.nTokens - 1; i++) {
            jj = this.tokenFinalIndices[i];
            if (Chars.isQuotationMark(this.sentenceCaseString.charAt(jj))) {
                jj--;
            }
            if (this.enteredString.charAt(jj) == '.') {
                kk = this.tokenInitialIndices[i + 1];
                if (Chars.isQuotationMark(this.sentenceCaseString.charAt(kk))) {
                    kk++;
                }
                if (!this.whiteSpaces[kk]) {
                    this.toUpperCase(kk);
                }
            }
        }
        return this.sentenceCaseString;
    }

    // Converts to sentence case
    // static method
    public static String toSentenceCase(String ss)  {
        Strings strs = new Strings(ss);
        return strs.toSentenceCase();
    }

    // Removes all accents and replaces the accented letter with its non-accented equivalent 
    // instance method
    public String removeAccents() {

        String ss = this.enteredString;
        int nSS = ss.length();

        // Remove html names and replace with unaccented equivalent
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsA; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(htmlNamesA[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + htmlNamesA[i].length();
                    String hold0 = ss.substring(0, pos0) + this.replSymbolsA[i];
                    if (pos1 < this.nEntered) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                } else {
                    test0 = false;
                }
            }
        }

        // Remove html numbers and replace with unaccented equivalent
        for (int i = 0; i < this.nSymbolsA; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(htmlNumbersA[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + htmlNumbersA[i].length();
                    String hold0 = ss.substring(0, pos0) + this.replSymbolsA[i];
                    if (pos1 < this.nEntered) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                } else {
                    test0 = false;
                }
            }
        }

        // Replace int equivalent with unaccented equivalent
        for (int i = 0; i < this.nSymbolsA; i++) {
            for (int j = 0; j < nSS; j++) {
                if ((int) ss.charAt(j) == this.decNumbersA[i]) {
                    ss = ss.substring(0, j) + this.replSymbolsA[i] + ss.substring(j + 1);
                }
            }
            nSS = ss.length();
        }

        this.unAccentedString = ss;
        return ss;
    }

    // Removes all accents and replaces the accented letter with its non-accented equivalent 
    // static method
    public static String removeAccents(String ss)  {
        Strings strs = new Strings(ss);
        return strs.removeAccents();
    }

    // TOKENISATION
    // Treat tab as tab, otherwise replace by white spave
    public void tabAsTab() {
        this.tabastab = true;
    }

    // Check presence of a separator
    public String checkForSeparator(char sepc) {
        boolean sepPresent = false;
        String seps = "" + sepc;
        if (sepc == '"') {
            seps = "\"";
        }
        seps = this.separatorCheck(seps);
        return seps;
    }

    // Required by tokenisation methods
    private String checkForSeparator(String seps) {
        seps = this.separatorCheck(seps);
        return seps;
    }

    // Checks tat separator is present and corrects its form if the separator is a metacharacter
    private String separatorCheck(String seps) {
        if (seps.equals("\\")) {
            throw new IllegalArgumentException("Method does not handle backslash \\ as separator");
        }

        // If separator is white space checck for tabs
        if (seps.equals(" ") && !this.tabastab) {
            if (this.enteredString.indexOf('\t') > -1) {
                this.editedString = this.enteredString;
                this.enteredString = this.enteredString.replace('\t', ' ');
                this.tabsReplaced = true;
            }
        }

        // chech if separator present
        if (this.enteredString.contains(seps)) {
            // Check if separator is a metacharacter
            for (int i = 0; i < this.nMeta; i++) {
                if (seps.equals(this.metacharactersS[i])) {
                    seps = "[" + seps + "}";
                    break;
                }
            }
        } else {
            throw new IllegalArgumentException("String " + this.enteredString + " does not contain " + seps);
        }
        return seps;
    }

    // Split the string into tokens with supplied separator 
    // instance methods
    // char separator
    public String[] tokens(char sepr) {
        String seps = this.checkForSeparator(sepr);
        return this.calculateTokens(seps);
    }

    // String separator
    public String[] tokens(String sepr) {
        String seps = this.checkForSeparator(sepr);
        return this.calculateTokens(seps);
    }

    // white space separator
    public String[] tokens() {
        String seps = " ";
        return this.calculateTokens(seps);
    }

    public String[] tokensWhiteSpace() {
        String seps = " ";
        return this.calculateTokens(seps);
    }

    // static methods
    // char separator
    public static String[] tokens(String string, char sepr)  {
        Strings sss = new Strings(string);
        return sss.tokens(sepr);
    }

    // String separator
    public static String[] tokens(String string, String sepr)  {
        Strings sss = new Strings(string);
        return sss.tokens(sepr);
    }

    // white space separator
    public static String[] tokensWhiteSpace(String string)  {
        Strings sss = new Strings(string);
        return sss.tokens();
    }
    
    // Checks for white spaces
    // instance method
    public boolean areThereWhiteSpaces() {
        boolean test0 = false;
        for (int i = 0; i < this.nSymbolsSQ; i++) {
            int ind[] = this.indicesOf(" ");
            if(ind!=null)test0 = true;
        }
         
        return test0;
    }
    
    // Checks for white spaces
    // static method
    public static boolean areThereWhiteSpaces(String ss)  {
        Strings strs = new Strings(ss);
        return strs.areThereWhiteSpaces();
    }
    
    
    public int[] indicesOfWhiteSpaces() {
        this.whiteSpaceIndices = null;
        if(this.areThereWhiteSpaces()){
            ArrayList<Integer> alws = new ArrayList<Integer>();
            for (int i = 0; i < this.nEntered; i++) {
                if (Character.isWhitespace(this.enteredString.charAt(i))) {
                    alws.add(i);
                }
            }
            this.nWhiteSpaces = alws.size();
            this.whiteSpaceIndices = new int[this.nWhiteSpaces];
            for(int i=0; i<this.nWhiteSpaces; i++){
                this.whiteSpaceIndices[i] = alws.get(i);
            }
        }
        return this.whiteSpaceIndices;
    }
      
    public static int[] indicesOfWhiteSpaces(String ss)  {
        Strings strs = new Strings(ss);
        return strs.indicesOfWhiteSpaces();
    }
    
    
     public boolean[] whiteSpaces() {
        this.whiteSpaces = new boolean[this.nEntered];
        boolean test = true;
        for (int i = 0; i < this.nEntered; i++) {
            whiteSpaces[i] = false;
            if (Character.isWhitespace(this.enteredString.charAt(i))) {
                whiteSpaces[i] = true;
            }
        }
        return this.whiteSpaces;
    }
      
    public static boolean[] whiteSpaces(String ss)  {
        Strings strs = new Strings(ss);
        return strs.whiteSpaces();
    }

    // Removes white spaces
    // instance method
    public String removeWhiteSpaces() {
        String ret = "";
        for (int i = 0; i < this.nAsciiString; i++) {
            char cc = this.asciiString.charAt(i);
            if (!Character.isWhitespace(cc)) {
                ret += cc;
            }
        }
        return ret.trim();
    }

    // Removes white spaves 
    // static method
    public static String removeWhiteSpaces(String line)  {
        Strings ss = new Strings(line);
        return ss.removeWhiteSpaces();
    }

 
    private String[] calculateTokens(String sepr) {
        
        this.tokensDone = false;
        // check separator is present and in an appropriate form
        this.separator = this.checkForSeparator(sepr);

        // Split string into tokens
        // split ino parts
        String[] parts = this.enteredString.split(this.separator);
        int nParts = parts.length;
        // if separator is a white space, remove empty parts, eg as caused by adjacent reeated separators
        ArrayList<String> altok = new ArrayList<String>();
        for (int i = 0; i < nParts; i++) {
            if(this.separator.equals(" ")){
                if (parts[i].equals("")) {
                    // ignore
                } else {
                    altok.add(parts[i]);
                }
            }
            else{
                altok.add(parts[i]);
            }
        }
        this.nTokens = altok.size();
        this.tokens = new String[this.nTokens];
        this.tokenInitialIndices = new int[this.nTokens];
        this.tokenFinalIndices = new int[this.nTokens];
        for (int i = 0; i < this.nTokens; i++) {
            this.tokens[i] = altok.get(i).trim();
        }

        // Find initial and final index of each token
        int toklen = -1;
        int istart = 0;
        int ii = 0;
        String shold = this.enteredString;
        for (int i = 0; i < this.nTokens; i++) {
            shold = shold.substring(ii);
            toklen = this.tokens[i].length();
            ii = shold.indexOf(tokens[i]);
            this.tokenInitialIndices[i] = ii + istart;
            this.tokenFinalIndices[i] = this.tokenInitialIndices[i] + toklen - 1;
            istart += ii;
        }

        if (this.tabsReplaced) {
            this.enteredString = this.editedString;
        }
        this.tokensDone = true;
        return this.tokens;
    }
    
    // Remove empty tokens
    public String[] returnNonEmptyTokens(){
        String[] retn = null;
        if(this.tokensDone){
            ArrayList<String> alne = new ArrayList<String>();
            for(int i=0; i<this.nTokens; i++){
                if(this.tokens[i].equals("")){
                    // ignore
                }
                else{
                    alne.add(this.tokens[i]);
                }
            }
            int net = alne.size();
            retn = new String[net];
            for(int j=0; j<net; j++){
                retn[j] = alne.get(j);
            }
        }
        else{
            System.out.println("Method: returnNonEmptyTokens,   no tokenisation method has been called");
        }
        return retn;
    }

    // Return token initial indices
    // instance method
    public int[] tokenInitialIndices() {
        if (!this.tokensDone) {
            this.tokens();
        }
        return this.tokenInitialIndices;
    }

    // Return token initial indices
    // static method
    public static int[] tokenInitialIndices(String ss)  {
        Strings strs = new Strings(ss);
        return strs.tokenInitialIndices();
    }

    // Return token final indices
    // instance method
    public int[] tokenFinalIndices() {
        if (!this.tokensDone) {
            this.tokens();
        }
        return this.tokenFinalIndices;
    }

    // Return token final indices
    // static method
    public static int[] tokenFinalIndices(String ss)  {
        Strings strs = new Strings(ss);
        return strs.tokenFinalIndices();
    }
    
    // Return all tokens fo all lines within a text file
    // static methods
    public static String[] tokensWithinTextFile(String filename, String separator) {
        Strings ssss = new Strings();
        ssss.fileEntered(filename);
        return ssss.tokens(separator);
    }
    
    public static String[] tokensWithinTextFile(String filename, char separator) {
        Strings ssss = new Strings();
        ssss.fileEntered(filename);
        return ssss.tokens(separator);
    }
    
    public static String[] tokensWithinTextFile(String filename) {
        Strings ssss = new Strings();
        ssss.fileEntered(filename);
        return ssss.tokens();
    }
    
    

    // Replace separator
    // Instance methods
    // line replacement
    public String replaceSeparator(char oldSep, char newSep) {
        boolean test = true;
        return this.replaceSeparator(test, String.valueOf(oldSep), String.valueOf(newSep));
    }

    public String replaceSeparator(String oldSep, String newSep) {
        boolean test = true;
        String newS = this.enteredString.replaceAll(oldSep, newSep);
        return this.replaceSeparator(test, oldSep, newSep);
    }

    private String replaceSeparator(boolean test, String oldSep, String newSep) {
        String newS = this.enteredString.replaceAll(oldSep, newSep);
        newS = this.replacement(newS);
        return newS;
    }

    // file replacement
    public void replaceFileSeparatorI(String fileName, char oldSep, char newSep) {
        this.replaceFileSeparatorI(fileName, String.valueOf(oldSep), String.valueOf(newSep));
    }

    public void replaceFileSeparatorI(String fileName, String oldSep, String newSep) {
        this.replaceFileSeparatorI(fileName, "-1", oldSep, newSep);
    }

    public void replaceFileSeparatorI(String fileNameIn, String fileNameOut, char oldSep, char newSep) {
        this.replaceFileSeparatorI(fileNameIn, fileNameOut, String.valueOf(oldSep), String.valueOf(newSep));
    }

    public void replaceFileSeparatorI(String fileNameIn, String fileNameOut, String oldSep, String newSep) {

        FileInput fin = new FileInput(fileNameIn);
        boolean testf = fin.fileFound();
        if (!testf) {
            throw new IllegalArgumentException("File " + fileNameIn + " cannot be found\nDid you add the correct extension?");
        } else {
            String stem = fin.getStemName();
            int nStem = stem.length();
            String ext = fileNameIn.substring(nStem);
            if (!ext.trim().equals(".txt")) {
                throw new IllegalArgumentException("File " + fileNameIn + " does not appear to be a txt file");
            }
            String fileOutName = stem + "Edited" + fileNameIn.substring(nStem);

            int n = fin.numberOfLines();
            String[] lines = new String[n];
            String hold = null;
            for (int i = 0; i < n; i++) {
                hold = fin.readLine();
                lines[i] = Strings.replaceSeparator(hold, oldSep, newSep);
            }
            if (fileNameOut.equals("-1")) {
                int opt = this.fileReplacement();
                if (opt == 0) {
                    // New file
                    fileOutName = Db.readLine("Enter new file name (remember any extension)", fileNameOut);
                } else {
                    // overwrite
                    fileNameOut = fileNameIn;

                }
            }
            FileOutput fout = new FileOutput(fileNameOut);
            for (int i = 0; i < n; i++) {
                fout.println(lines[i]);
            }
        }
    }

    // Static methods
    // line replacement
    public static String replaceSeparator(String ss, char oldSep, char newSep)  {
        Strings sss = new Strings(ss);
        boolean test = false;
        return sss.replaceSeparator(test, String.valueOf(oldSep), String.valueOf(newSep));
    }

    public static String replaceSeparator(String ss, String oldSep, String newSep)  {
        Strings sss = new Strings(ss);
        boolean test = false;
        return sss.replaceSeparator(test, oldSep, newSep);
    }

    // file replacement
    public static void replaceFileSeparatorS(String fileNameIn, String fileNameOut, char oldSep, char newSep)  {
        Strings.replaceFileSeparatorS(fileNameIn, fileNameOut, String.valueOf(oldSep), String.valueOf(newSep));
    }

    public static void replaceFileSeparatorS(String fileNameIn, String fileNameOut, String oldSep, String newSep)  {
        Strings sss = new Strings();
        sss.replaceFileSeparatorI(fileNameIn, fileNameOut, oldSep, newSep);
    }

    public static void replaceFileSeparatorS(String fileName, char oldSep, char newSep)   {
        Strings.replaceFileSeparatorS(fileName, String.valueOf(oldSep), String.valueOf(newSep));
    }

    public static void replaceFileSeparatorS(String fileName, String oldSep, String newSep)  {
        Strings sss = new Strings();
        sss.replaceFileSeparatorI(fileName, oldSep, newSep);
    }
    

    // Required be replaceSeparator methods
    private String replacement(String newS)   {
        if (this.suppressDialogue) {
            this.editedString = newS;
        } else {
            String headerComment = "Do you wish to:";
            String[] comments = {"Store as the edited String?", "Replace the current entered String?"};
            String[] boxTitles = {"Edited String", "Replace entered String"};
            int defaultBox = 1;
            int opt = Db.optionBox(headerComment, comments, boxTitles, defaultBox);
            if (opt == 0) {
                this.editedString = newS;
            } else {
                this.enteredString = newS;
            }
        }
        return newS;
    }

    // Required be replaceFileSeparator methods
    private int fileReplacement() {
        String headerComment = "Do you wish to:";
        String[] comments = {"Create a new file?", "Replace the current file?"};
        String[] boxTitles = {"New file", "Replace old file"};
        int defaultBox = 1;
        int opt = Db.optionBox(headerComment, comments, boxTitles, defaultBox);
        return opt;
    }

    // Split the string into principal tokens
    // instance method
    public String[] principalTokens() {
        if (!this.tokensDone) {
            this.tokens();
        }
        ArrayList<Integer> al = new ArrayList<Integer>();
        al.add(0);

        for (int i = 1; i < this.nTokens; i++) {
            String tt = Strings.removeQuotationMarks(this.tokens[i]);
            boolean test = true;
            for (int j = 0; j < this.nArticles; j++) {
                if (tt.equalsIgnoreCase(this.articles[j])) {
                    test = false;
                    break;
                }
            }
            if (test) {
                for (int j = 0; j < this.nPrepositions; j++) {
                    if (tt.equalsIgnoreCase(this.prepositions[j])) {
                        test = false;
                        break;
                    }
                }
            }
            if (test) {
                for (int j = 0; j < this.nConjunctions; j++) {
                    if (tt.equalsIgnoreCase(this.conjunctions[j])) {
                        test = false;
                        break;
                    }
                }
            }
            if (test) {
                al.add(i);
            }
        }

        this.nPrincipalTokens = al.size();
        this.principalTokens = new String[this.nPrincipalTokens];
        this.principalTokenInitialIndices = new int[this.nPrincipalTokens];
        this.principalTokenFinalIndices = new int[this.nPrincipalTokens];
        for (int i = 0; i < this.nPrincipalTokens; i++) {
            int j = al.get(i);
            this.principalTokens[i] = this.tokens[j];
            this.principalTokenInitialIndices[i] = this.tokenInitialIndices[j];
            this.principalTokenFinalIndices[i] = this.tokenFinalIndices[j];
        }

        return this.principalTokens;
    }

    // Split the string into principal tokens
    // static method
    public static String[] principalTokens(String ss) {
        Strings strs = new Strings(ss);
        return strs.principalTokens();
    }

    // Split the string into principal tokens ignoring obligatory  initial principal token
    // instance method
    public String[] principalTokensZero() {
        if (!this.tokensDone) {
            this.tokens();
        }
        ArrayList<Integer> al = new ArrayList<Integer>();

        for (int i = 0; i < this.nTokens; i++) {
            String tt = Strings.removeQuotationMarks(this.tokens[i]);
            boolean test = true;
            for (int j = 0; j < this.nArticles; j++) {
                if (tt.equalsIgnoreCase(this.articles[j])) {
                    test = false;
                    break;
                }
            }
            if (test) {
                for (int j = 0; j < this.nPrepositions; j++) {
                    if (tt.equalsIgnoreCase(this.prepositions[j])) {
                        test = false;
                        break;
                    }
                }
            }
            if (test) {
                for (int j = 0; j < this.nConjunctions; j++) {
                    if (tt.equalsIgnoreCase(this.conjunctions[j])) {
                        test = false;
                        break;
                    }
                }
            }
            if (test) {
                al.add(i);
            }
        }

        this.nPrincipalTokens = al.size();
        this.principalTokens = new String[this.nPrincipalTokens];
        this.principalTokenInitialIndices = new int[this.nPrincipalTokens];
        this.principalTokenFinalIndices = new int[this.nPrincipalTokens];
        for (int i = 0; i < this.nPrincipalTokens; i++) {
            int j = al.get(i);
            this.principalTokens[i] = this.tokens[j];
            this.principalTokenInitialIndices[i] = this.tokenInitialIndices[j];
            this.principalTokenFinalIndices[i] = this.tokenFinalIndices[j];
        }

        return this.principalTokens;
    }

    // Split the string into principal tokens ignoring obligatory  initial principal token
    // static method
    public static String[] principalTokensZero(String ss)   {
        Strings strs = new Strings(ss);
        return strs.principalTokensZero();
    }

    // CONCATENATION
    // instance methods
    public String concat(String[] strings) {
        int n = strings.length;
        String con = this.editedString;
        for (int i = 0; i < n; i++) {
            con = con.concat(strings[i]);
        }
        return con;
    }

    public String concat(String string) {
        String con = this.editedString;
        con = con.concat(string);
        return con;
    }

    // static methods
    public static String concat(String ss, String[] strings) {
        int n = strings.length;
        String con = ss;
        for (int i = 0; i < n; i++) {
            con = con.concat(strings[i]);
        }
        return con;
    }

    public static String concat(String string1, String string2) {
        return string1.concat(string2);
    }

    public static String concatS(String[] strings) {
        int n = strings.length;
        String con = strings[0];
        for (int i = 1; i < n; i++) {
            con = con.concat(strings[i]);
        }
        return con;
    }

    // ANALYSIS
    // RReturn all indices of a string or chacter
    // instace methods
    public int[] indicesOf(char cc) {
        return this.indicesOf(String.valueOf(cc));

    }

    public int[] indicesOf(String ss) {
        String sss = this.enteredString;
        ArrayList<Integer> alif = new ArrayList<Integer>();
        boolean test = true;
        int point = 0;
        int[] indss = null;
        int nio = 0;
        while (test) {
            int pos = sss.indexOf(ss);
            if (pos != -1) {
                nio++;
                point = point + pos;
                alif.add(point);
                sss = sss.substring(pos + 1);
                point += 1;
                if (point >= this.nEntered) {
                    test = false;
                }
            } else {
                test = false;
            }
        }
        if (nio > 0) {
            int nI = alif.size();
            indss = new int[nI];
            for (int i = 0; i < nI; i++) {
                indss[i] = alif.get(i);
            }
        }
        return indss;
    }

    // static methods
    public static int[] indicesOf(char cc, String ss) {
        return Strings.indicesOf(String.valueOf(cc), ss);
    }

    public static int[] indicesOf(String cc, String ss) {
        Strings sss = new Strings(ss);
        return sss.indicesOf(cc);
    }

    // Return ASCII decimal equivalents
    // Instance method
    public int[] getAsciiDecimal() {
        if(this.asciiDecimalString==null){
            this.asciiDecimalString = new int[this.nEntered];
            for (int i = 0; i < nEntered; i++) {
                this.asciiDecimalString[i] = Chars.getASCIIdec(this.enteredString.charAt(i));
            }  
        }
        return this.asciiDecimalString;
    }

    // Static method
    public static int[] getAsciiDecimal(String ss) {
        Strings sss = new Strings(ss);
        return sss.getAsciiDecimal();
    }

    // Return ASCII hexadecimal equivalents
    // Instance method
    public String[] getAsciiHexadecimal() {
        String[] asciiHex = new String[this.nEntered];
        for (int i = 0; i < nEntered; i++) {
            asciiHex[i] = Chars.getASCIIhex(this.enteredString.charAt(i));
        }
        return asciiHex;
    }

    // Static method
    public static String[] getAsciiHexadecimal(String ss) {
        Strings sss = new Strings(ss);
        return sss.getAsciiHexadecimal();
    }

    // TESTS
    // Check if all characters are alphabetic
    // instance method
    public boolean areAlphabetic() {
        boolean test0 = true;
        char cc = '\u0000';
        for (int i = 0; i < this.nEntered; i++) {
            cc = this.enteredString.charAt(i);
            test0 = Character.isAlphabetic(cc);
            if (!test0) {
                break;
            }
        }
        return test0;
    }

    // static method
    public static boolean areAlphabetic(String ss) {
        Strings sss = new Strings(ss);
        return sss.areAlphabetic();
    }

    // Check if all characters are decimal digits
    // instance method
    public boolean areDigits() {
        return this.areDecimalDigits();
    }
    
    
    public boolean areDecimalDigits() {
        boolean test0 = true;
        char cc = '\u0000';
        for (int i = 0; i < this.nEntered; i++) {
            cc = this.enteredString.charAt(i);
            test0 = Character.isDigit(cc);
            if (!test0) {
                break;
            }
        }
        return test0;
    }

    // static method
    public static boolean areDigits(String ss) {
         return Strings.areDecimalDigits(ss);
    }
     
    public static boolean 
        areDecimalDigits(String ss) {
        Strings sss = new Strings(ss);
        return sss.areDigits();
    }
        
    //  Check if all characters are hexadecimal digits
    // instace method
    public boolean areHexadecimalDigits(){
        char[] ccdd = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
        boolean test0 = true;
        boolean test1 = false;
        String ss = this.enteredString;
        for(int i=0; i<this.nSymbolsAll; i++){
            char cc = ss.charAt(i);
            test1 = false;
            for(int j=0; j<16; j++){
                if(cc==ccdd[j]){
                    test1 = true;
                }
                if(test1)break;
            }
            if(!test1){
                test0 = false;
            }
            if(!test0)break;
        }
        return test0;
    }
    
    // static method
    public static boolean areHexadecimalDigits(String ss){
    char[] ccdd = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
        boolean test0 = true;
        boolean test1 = false;
        int nl = ss.length();
        for(int i=0; i<nl; i++){
            char cc = ss.charAt(i);
            test1 = false;
            for(int j=0; j<16; j++){
                if(cc==ccdd[j]){
                    test1 = true;
                }
                if(test1)break;
            }
            if(!test1){
                test0 = false;
            }
            if(!test0)break;
        }
        return test0;
    }
   
    //  Check if all characters are octal digits
    // instace method
    public boolean areOctalDigits(){
        char[] ccdd = {'0', '1', '2', '3', '4', '5', '6', '7'};
        boolean test0 = true;
        boolean test1 = false;
        String ss = this.enteredString;
        for(int i=0; i<this.nSymbolsAll; i++){
            char cc = ss.charAt(i);
            test1 = false;
            for(int j=0; j<16; j++){
                if(cc==ccdd[j]){
                    test1 = true;
                }
                if(test1)break;
            }
            if(!test1){
                test0 = false;
            }
            if(!test0)break;
        }
        return test0;
    }
    
    // static method
    public static boolean areOctalDigits(String ss){
    char[] ccdd = {'0', '1', '2', '3', '4', '5', '6', '7'};
        boolean test0 = true;
        boolean test1 = false;
        int nl = ss.length();
        for(int i=0; i<nl; i++){
            char cc = ss.charAt(i);
            test1 = false;
            for(int j=0; j<16; j++){
                if(cc==ccdd[j]){
                    test1 = true;
                }
                if(test1)break;
            }
            if(!test1){
                test0 = false;
            }
            if(!test0)break;
        }
        return test0;
    }
    
     //  Check if all characters are binary digits
    // instace method
    public boolean areBinaryDigits(){
        char[] ccdd = {'0', '1'};
        boolean test0 = true;
        boolean test1 = false;
        String ss = this.enteredString;
        for(int i=0; i<this.nSymbolsAll; i++){
            char cc = ss.charAt(i);
            test1 = false;
            for(int j=0; j<16; j++){
                if(cc==ccdd[j]){
                    test1 = true;
                }
                if(test1)break;
            }
            if(!test1){
                test0 = false;
            }
            if(!test0)break;
        }
        return test0;
    }
    
    // static method
    public static boolean areBinaryDigits(String ss){
    char[] ccdd = {'0', '1'};
        boolean test0 = true;
        boolean test1 = false;
        int nl = ss.length();
        for(int i=0; i<nl; i++){
            char cc = ss.charAt(i);
            test1 = false;
            for(int j=0; j<16; j++){
                if(cc==ccdd[j]){
                    test1 = true;
                }
                if(test1)break;
            }
            if(!test1){
                test0 = false;
            }
            if(!test0)break;
        }
        return test0;
    }
    
    // PREPARATION FOR HTML DISPLAY
    // Conver tHtml Names To Html Numbers
    public String convertHtmlNamesToHtmlNumbers() {

        String ss = this.enteredString;
        for (int i = 0; i < this.nSymbolsAll; i++) {

        }
        return null;
    }

    // check if a code is an html number code
    // Returns true if code is, false if not
    private boolean checkIfHtmlNumberCode(String code) {
        boolean test0 = true;
        code = code.trim();
        int nc = code.length();
        if ((int) code.charAt(0) != 38) {
            test0 = false;
        }
        if ((int) code.charAt(1) != 35) {
            test0 = false;
        }
        if ((int) code.charAt(nc - 1) != 59) {
            test0 = false;
        }
        String num = code.substring(2, nc - 1);
        int nss = num.length();
        for (int i = 0; i < nss; i++) {
            if (!Chars.isDigit(num.charAt(i))) {
                test0 = false;
            }
        }
        return test0;
    }

    //Convert all dashes to html number code representation 
    // instance method
    public String dashesToHtmlNumberCodes() {

        String ss = this.enteredString;
        int nSS = ss.length();

        // Convert html names to html numbers
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsD; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNamesD[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNamesD[i].length();
                    String hold0 = ss.substring(0, pos0) + this.htmlNumbersD[i];
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert int equivalents to html numbers 
        for (int i = 0; i < nSS; i++) {
            int cc = (int) ss.charAt(i);
            for (int j = 0; j < this.nSymbolsD; j++) {
                if (cc == this.decNumbersD[j]) {
                    String hold1 = ss.substring(0, i) + this.htmlNumbersD[j];
                    if (i + 1 < nSS) {
                        hold1 += ss.substring(i + 1);
                    }
                    ss = hold1;
                    nSS = ss.length();
                    break;
                }
            }
        }
        return ss;
    }

    // Convert all dashes to html number representation 
    // static method
    public static String dashesToHtmlNumberCodes(String ss) {
        Strings strs = new Strings(ss);
        return strs.dashesToHtmlNumberCodes();
    }

    //Convert all dashes to a specific html number code representation 
    // instance method
    public String dashesToSpecificHtmlNumberCode(String code) {

        // check code is html code
        boolean test1 = this.checkIfHtmlNumberCode(code);
        if (!test1) {
            throw new IllegalArgumentException("Argument," + code + ", does not appear to be a html number code");
        }

        String ss = this.enteredString;
        int nSS = ss.length();

        // Convert html names to specific html number code
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsD; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNamesD[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNamesD[i].length();
                    String hold0 = ss.substring(0, pos0) + code;
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert html number codes to specific html number code
        test0 = true;
        pos0 = -1;
        pos1 = -1;
        for (int i = 0; i < this.nSymbolsD; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNumbersD[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNumbersD[i].length();
                    String hold0 = ss.substring(0, pos0) + code;
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert int equivalents to html numbers 
        for (int i = 0; i < nSS; i++) {
            int cc = (int) ss.charAt(i);
            for (int j = 0; j < this.nSymbolsD; j++) {
                if (cc == this.decNumbersD[j]) {
                    String hold1 = ss.substring(0, i) + code;
                    if (i + 1 < nSS) {
                        hold1 += ss.substring(i + 1);
                    }
                    ss = hold1;
                    nSS = ss.length();
                    break;
                }
            }
        }
        return ss;
    }

    // Convert all dashes to a specific html number code representation 
    // static method
    public static String dashesToSpecificHtmlNumberCode(String code, String ss) {
        Strings strs = new Strings(ss);
        return strs.dashesToSpecificHtmlNumberCode(code);
    }

    // Convert all accented characters to html number representation 
    // instance method
    public String accentedToHtmlCodes() {
        return this.accentsToHtmlNumbers();
    }

    // originally titled method (used in TC updatee
    public String accentsToHtmlNumbers() {

        String ss = this.enteredString;
        int nSS = ss.length();

        // Convert html names to html numbers
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsA; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNamesA[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNamesA[i].length();
                    String hold0 = ss.substring(0, pos0) + this.htmlNumbersA[i];
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert int equivalents to html numbers 
        for (int i = 0; i < nSS; i++) {
            int cc = (int) ss.charAt(i);
            for (int j = 0; j < this.nSymbolsA; j++) {
                if (cc == this.decNumbersA[j]) {
                    String hold1 = ss.substring(0, i) + this.htmlNumbersA[j];
                    if (i + 1 < nSS) {
                        hold1 += ss.substring(i + 1);
                    }
                    ss = hold1;
                    nSS = ss.length();
                    break;
                }
            }
        }

        return ss;
    }

    // Convert all accented characters to html number representation 
    // static method
    public static String accentsToHtmlNumbers(String ss) {
        Strings strs = new Strings(ss);
        return strs.accentsToHtmlNumbers();
    }

    //Convert all single quoatation marks to html number code representation 
    // instance method
    public String singleQuotationMarksToHtmlNumberCodes() {

        String ss = this.enteredString;
        int nSS = ss.length();

        // Convert html names to html numbers
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsSQ; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNamesSQ[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNamesSQ[i].length();
                    String hold0 = ss.substring(0, pos0) + this.htmlNumbersSQ[i];
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert int equivalents to html numbers 
        for (int i = 0; i < nSS; i++) {
            int cc = (int) ss.charAt(i);
            for (int j = 0; j < this.nSymbolsSQ; j++) {
                if (cc == this.decNumbersSQ[j]) {
                    String hold1 = ss.substring(0, i) + this.htmlNumbersSQ[j];
                    if (i + 1 < nSS) {
                        hold1 += ss.substring(i + 1);
                    }
                    ss = hold1;
                    nSS = ss.length();
                    break;
                }
            }
        }
        return ss;
    }

    // Convert all single quotation marks to html number representation 
    // static method
    public static String singleQuotationMarksToHtmlNumberCodes(String ss) {
        Strings strs = new Strings(ss);
        return strs.singleQuotationMarksToHtmlNumberCodes();
    }

    //Convert all single quotation marks to a specific html number code representation 
    // instance method
    public String singleQuotationMarksToSpecificHtmlNumberCode(String code) {

        // check code is html code
        boolean test1 = this.checkIfHtmlNumberCode(code);
        if (!test1) {
            throw new IllegalArgumentException("Argument," + code + ", does not appear to be a html number code");
        }

        String ss = this.enteredString;
        int nSS = ss.length();

        // Convert html names to specific html number code
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsSQ; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNamesSQ[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNamesSQ[i].length();
                    String hold0 = ss.substring(0, pos0) + code;
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert html number codes to specific html number code
        test0 = true;
        pos0 = -1;
        pos1 = -1;
        for (int i = 0; i < this.nSymbolsSQ; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNumbersSQ[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNumbersSQ[i].length();
                    String hold0 = ss.substring(0, pos0) + code;
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert int equivalents to html numbers 
        for (int i = 0; i < nSS; i++) {
            int cc = (int) ss.charAt(i);
            for (int j = 0; j < this.nSymbolsSQ; j++) {
                if (cc == this.decNumbersSQ[j]) {
                    String hold1 = ss.substring(0, i) + code;
                    if (i + 1 < nSS) {
                        hold1 += ss.substring(i + 1);
                    }
                    ss = hold1;
                    nSS = ss.length();
                    break;
                }
            }
        }
        return ss;
    }

    // Convert all single quotation mark to a specific html number code representation 
    // static method
    public static String singleQuotationMarksToSpecificHtmlNumberCode(String code, String ss) {
        Strings strs = new Strings(ss);
        return strs.singleQuotationMarksToSpecificHtmlNumberCode(code);
    }

    // Return array of single quotation mark numbers
    // instance method
    public String[] getSingleQuotationMarkHtmlNumbers() {
        return this.htmlNumbersSQ;
    }

    //Convert all double quoatation marks to html number code representation 
    // instance method
    public String doubleQuotationMarksToHtmlNumberCodes() {

        String ss = this.enteredString;
        int nSS = ss.length();

        // Convert html names to html numbers
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsDQ; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNamesDQ[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNamesDQ[i].length();
                    String hold0 = ss.substring(0, pos0) + this.htmlNumbersDQ[i];
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert int equivalents to html numbers 
        for (int i = 0; i < nSS; i++) {
            int cc = (int) ss.charAt(i);
            for (int j = 0; j < this.nSymbolsDQ; j++) {
                if (cc == this.decNumbersDQ[j]) {
                    String hold1 = ss.substring(0, i) + this.htmlNumbersDQ[j];
                    if (i + 1 < nSS) {
                        hold1 += ss.substring(i + 1);
                    }
                    ss = hold1;
                    nSS = ss.length();
                    break;
                }
            }
        }
        return ss;
    }

    // Convert all double quotation marks to html number representation 
    // static method
    public static String doubleQuotationMarksToHtmlNumberCodes(String ss) {
        Strings strs = new Strings(ss);
        return strs.doubleQuotationMarksToHtmlNumberCodes();
    }

    //Convert all double quotation marks to a specific html number code representation 
    // instance method
    public String doubleQuotationMarksToSpecificHtmlNumberCode(String code) {

        // check code is html code
        boolean test1 = this.checkIfHtmlNumberCode(code);
        if (!test1) {
            throw new IllegalArgumentException("Argument," + code + ", does not appear to be a html number code");
        }

        String ss = this.enteredString;
        int nSS = ss.length();

        // Convert html names to specific html number code
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsDQ; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNamesDQ[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNamesDQ[i].length();
                    String hold0 = ss.substring(0, pos0) + code;
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert html number codes to specific html number code
        test0 = true;
        pos0 = -1;
        pos1 = -1;
        for (int i = 0; i < this.nSymbolsDQ; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNumbersDQ[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNumbersDQ[i].length();
                    String hold0 = ss.substring(0, pos0) + code;
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert int equivalents to html numbers 
        for (int i = 0; i < nSS; i++) {
            int cc = (int) ss.charAt(i);
            for (int j = 0; j < this.nSymbolsDQ; j++) {
                if (cc == this.decNumbersDQ[j]) {
                    String hold1 = ss.substring(0, i) + code;
                    if (i + 1 < nSS) {
                        hold1 += ss.substring(i + 1);
                    }
                    ss = hold1;
                    nSS = ss.length();
                    break;
                }
            }
        }
        return ss;
    }

    // Convert all double quotation marks to a specific html number code representation 
    // static method
    public static String doubleQuotationMarksToSpecificHtmlNumberCode(String code, String ss) {
        Strings strs = new Strings(ss);
        return strs.doubleQuotationMarksToSpecificHtmlNumberCode(code);
    }

    // Convert all quotation mark to html number representation 
    // instance method
    public String allQuotationMarksToHtmlNumbers() {
        return this.quotationMarksToHtmlNumbers();
    }

    public String quotationMarksToHtmlNumbers() {

        String ss = this.enteredString;
        int nSS = ss.length();

        // Convert html names to html numbers
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsQ; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNamesQ[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNamesQ[i].length();
                    String hold0 = ss.substring(0, pos0) + this.htmlNumbersQ[i];
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert int equivalents to html numbers 
        for (int i = 0; i < nSS; i++) {
            int cc = (int) ss.charAt(i);
            for (int j = 0; j < this.nSymbolsQ; j++) {
                if (cc == this.decNumbersQ[j]) {
                    String hold1 = ss.substring(0, i) + this.htmlNumbersQ[j];
                    if (i + 1 < this.nEntered) {
                        hold1 += ss.substring(i + 1);
                    }
                    ss = hold1;
                    nSS = ss.length();
                    break;
                }
            }
        }

        return ss;
    }

    // Convert all quotation marks to html number representation except "
    // instance method
    // REplaces " html code by "\22
    // For update program
    public String quotationMarksToHtmlNumbersD() {
        String ss = quotationMarksToHtmlNumbers();

        boolean test = true;
        String testString = "&#34;";
        while (test) {
            int pos = ss.indexOf(testString);
            if (pos == -1) {
                test = false;
            } else {
                ss = ss.substring(0, pos) + "\"" + ss.substring(pos + 5);
            }
        }
        return ss;
    }

    // Convert all quotation mark to html number representation 
    // static method
    public static String allQuotationMarksToHtmlNumbers(String ss) {
        return Strings.quotationMarksToHtmlNumbers(ss);
    }

    public static String quotationMarksToHtmlNumbers(String ss) {
        Strings strs = new Strings(ss);
        return strs.quotationMarksToHtmlNumbers();
    }

    // Convert all quotation mark to html number representation except "
    // static method
    public static String quotationMarksToHtmlNumbersD(String ss) {
        Strings strs = new Strings(ss);
        return strs.quotationMarksToHtmlNumbersD();
    }

    //Convert all double q marks to a specific html number code representation 
    // instance method
    public String allQuotationMarksToSpecificHtmlNumberCode(String code) {

        // check code is html code
        boolean test1 = this.checkIfHtmlNumberCode(code);
        if (!test1) {
            throw new IllegalArgumentException("Argument," + code + ", does not appear to be a html number code");
        }

        String ss = this.enteredString;
        int nSS = ss.length();

        // Convert html names to specific html number code
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsQ; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNamesQ[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNamesQ[i].length();
                    String hold0 = ss.substring(0, pos0) + code;
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert html number codes to specific html number code
        test0 = true;
        pos0 = -1;
        pos1 = -1;
        for (int i = 0; i < this.nSymbolsQ; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(this.htmlNumbersQ[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + this.htmlNumbersQ[i].length();
                    String hold0 = ss.substring(0, pos0) + code;
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                } else {
                    test0 = false;
                }
            }
        }

        // Convert int equivalents to html numbers 
        for (int i = 0; i < nSS; i++) {
            int cc = (int) ss.charAt(i);
            for (int j = 0; j < this.nSymbolsQ; j++) {
                if (cc == this.decNumbersQ[j]) {
                    String hold1 = ss.substring(0, i) + code;
                    if (i + 1 < nSS) {
                        hold1 += ss.substring(i + 1);
                    }
                    ss = hold1;
                    nSS = ss.length();
                    break;
                }
            }
        }
        return ss;
    }

    // Convert all quotation marks to a specific html number code representation 
    // static method
    public static String allQuotationMarksToSpecificHtmlNumberCode(String code, String ss) {
        Strings strs = new Strings(ss);
        return strs.allQuotationMarksToSpecificHtmlNumberCode(code);
    }

    // Convert all accent html name and html number representations to decimal integer representation 
    // instance method
    public String accentsToDec() {

        String ss = this.enteredString;
        int nSS = ss.length();

        // Convert html names 
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsA; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(htmlNamesA[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + htmlNamesA[i].length();
                    String hold0 = ss.substring(0, pos0) + (char) this.decNumbersA[i];
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                } else {
                    test0 = false;
                }
            }
        }

        // Convert html numbers 
        for (int i = 0; i < this.nSymbolsA; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(htmlNumbersA[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + htmlNumbersA[i].length();
                    String hold0 = ss.substring(0, pos0) + (char) this.decNumbersA[i];
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                } else {
                    test0 = false;
                }
            }
        }

        return ss;
    }

    // Convert all quotation mark html name and html number representations to decimal integer representation 
    // static method
    public static String accentsToDec(String ss) {
        Strings strs = new Strings(ss);
        return strs.accentsToDec();
    }

    // Convert all dash html name and html number representations to decimal integer representation 
    // instance method
    public String dashesToDec() {

        String ss = this.enteredString;
        int nSS = ss.length();

        // Convert html names 
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsD; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(htmlNamesD[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + htmlNamesD[i].length();
                    String hold0 = ss.substring(0, pos0) + (char) this.decNumbersD[i];
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                } else {
                    test0 = false;
                }
            }
        }

        // Convert html numbers 
        for (int i = 0; i < this.nSymbolsD; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(htmlNumbersD[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + htmlNumbersD[i].length();
                    String hold0 = ss.substring(0, pos0) + (char) this.decNumbersD[i];
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                } else {
                    test0 = false;
                }
            }
        }

        return ss;
    }

    // Convert all dash html name and html number representations to decimal integer representation 
    // static method
    public static String dashesToDec(String ss) {
        Strings strs = new Strings(ss);
        return strs.dashesToDec();
    }

    // Convert all quotation mark html name and html number representations to decimal integer representation 
    // instance method
    public String quotationMarksToDec() {

        String ss = this.enteredString;
        int nSS = ss.length();

        // Convert html names 
        boolean test0 = true;
        int pos0 = -1;
        int pos1 = -1;
        for (int i = 0; i < this.nSymbolsQ; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(htmlNamesQ[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + htmlNamesQ[i].length();
                    String hold0 = ss.substring(0, pos0) + (char) this.decNumbersQ[i];
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                } else {
                    test0 = false;
                }
            }
        }

        // Convert html numbers 
        for (int i = 0; i < this.nSymbolsQ; i++) {
            test0 = true;
            while (test0) {
                pos0 = ss.indexOf(htmlNumbersQ[i]);
                if (pos0 != -1) {
                    pos1 = pos0 + htmlNumbersQ[i].length();
                    String hold0 = ss.substring(0, pos0) + (char) this.decNumbersQ[i];
                    if (pos1 < nSS) {
                        hold0 += ss.substring(pos1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                } else {
                    test0 = false;
                }
            }
        }

        return ss;
    }

    // Convert all quotation mark html name and html number representations to decimal integer representation 
    // static method
    public static String quotationMarksToDec(String ss) {
        Strings strs = new Strings(ss);
        return strs.quotationMarksToDec();
    }

    // Convert all quotation mark integer dec representations to html name representation 
    // instance method
    public String quotationMarksDecToName() {

        // Convert to html names 
        String ss = this.enteredString;
        int nSS = ss.length();

        for (int i = 0; i < nSS; i++) {
            int cc = (int) ss.charAt(i);
            for (int j = 0; j < this.nSymbolsQ; j++) {
                if (cc == this.decNumbersQ[j]) {
                    String hold0 = ss.substring(0, i) + this.htmlNamesQ[j];
                    if (i < nSS - 1) {
                        hold0 += ss.substring(i + 1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                }
            }
        }
        return ss;
    }

    // Convert all quotation mark integer dec representations to html name representation 
    // static method
    public static String quotationMarksDecToName(String ss) {
        Strings strs = new Strings(ss);
        return strs.quotationMarksDecToName();
    }

    // Convert all quotation mark integer dec representations to html number representation 
    // instance method
    public String quotationMarksDecToHtmlNumber() {

        // Convert to html names 
        String ss = this.enteredString;
        int nSS = ss.length();

        for (int i = 0; i < nSS; i++) {
            int cc = (int) ss.charAt(i);
            for (int j = 0; j < this.nSymbolsQ; j++) {
                if (cc == this.decNumbersQ[j]) {
                    String hold0 = ss.substring(0, i) + this.htmlNumbersQ[j];
                    if (i < nSS - 1) {
                        hold0 += ss.substring(i + 1);
                    }
                    ss = hold0;
                    nSS = ss.length();
                    break;
                }
            }
        }
        return ss;
    }

    // Convert all quotation mark integer dec representations to html name representation 
    // static method
    public static String quotationMarksDecToHtmlNumber(String ss) {
        Strings strs = new Strings(ss);
        return strs.quotationMarksDecToHtmlNumber();
    }

   
  
  
  

    // Display integer character dASCII decimal equivalents
    // instance method
    public void displayAsciiDecimal() {
            this.displayIntegers();
    }
    
    public int[] displayIntegers() {
        int[] ints = new int[this.nEntered];
        for (int i = 0; i < this.nEntered; i++) {
            ints[i] = (int) this.enteredString.charAt(i);
        }
        PrintToScreen.print(ints);

        return ints;
    }

    // static method
    public void displayAsciiDecimal(String ss) {
        Strings.displayIntegers(ss);
    }
     
    public static int[] displayIntegers(String ss) {
        Strings strs = new Strings(ss);
        return strs.displayIntegers();
    }

    // Return index of the first occurence of a dash
    // instance method
    public int indexOfDash() {
        int ii = -1;
        int iis = -1;

        boolean found = false;
        for (int i = 0; i < this.nEntered; i++) {
            String cc = this.enteredString.substring(i, i + 1);
            for (int j = 0; j < this.nSymbolsD; j++) {
                if (cc.equals(symbolsD[j])) {
                    ii = i;
                    iis = i;
                    found = true;
                    break;
                }
            }
            if (found) {
                break;
            }
        }

        String[] namesD = {"&ndash;", "&mdash;", "&oline;"};
        int pos0 = -1;
        int iiss = -1;
        for (int j = 0; j < 3; j++) {
            pos0 = this.enteredString.indexOf(namesD[j]);
            if (pos0 != -1) {
                ii = pos0;
                if (iiss == -1) {
                    iiss = pos0;
                } else {
                    if (iiss < pos0) {
                        ii = iiss;
                    } else {
                        iiss = pos0;
                    }
                }
            }
        }
        if (iis != -1) {
            if (iiss != -1) {
                ii = Math.min(iis, iiss);
                iis = ii;
            } else {
                // no action
            }
        } else {
            if (iiss != -1) {
                ii = iiss;
                iis = iiss;
            } else {
                // no action 
            }
        }

        pos0 = -1;
        iiss = -1;
        for (int j = 0; j < this.nSymbolsD; j++) {
            pos0 = this.enteredString.indexOf(htmlNumbersD[j]);
            if (pos0 != -1) {
                ii = pos0;
                if (iiss == -1) {
                    iiss = pos0;
                } else {
                    if (iiss < pos0) {
                        ii = iiss;
                    } else {
                        iiss = pos0;
                    }
                }
            }
        }
        if (iis != -1) {
            if (iiss != -1) {
                ii = Math.min(iis, iiss);
                iis = ii;
            } else {
                // no action
            }
        } else {
            if (iiss != -1) {
                ii = iiss;
                iis = iiss;
            } else {
                // no action 
            }
        }

        return ii;
    }

    // Return index of the first occurence of a dash
    // static method
    public static int indexOfDash(String ss) {
        Strings strs = new Strings(ss);
        return strs.indexOfDash();
    }

    // Returns true if a percentage of the shorter string is common to both strings
    // The length of the putative common string is the length of the shorter string times identicalFraction rounded to the nearest integer 
    // instance method
    public boolean nearlyEquals(String ss1, double identicalPerCentage) {
        return this.nearlyEqualsF(ss1, identicalPerCentage / 100.0);
    }

    // Returns true if a percentage of the shorter string is common to both strings
    // The length of the putative common string is the length of the shorter string times identicalFraction rounded to the nearest integer 
    // static  method
    public static boolean nearlyEquals(String ss0, String ss1, double identicalPercentage) {
        Strings ss = new Strings(ss0);
        return ss.nearlyEquals(ss1, identicalPercentage);
    }

    // Returns true if 90% of the shorter string is common to both strings
    // The length of the putative common string is the length of the shorter string times identicalFraction rounded to the nearest integer 
    // instance method
    public boolean nearlyEquals(String ss1) {
        return this.nearlyEqualsF(ss1, 0.9);
    }

    // Returns true if 90% of the shorter string is common to both strings
    // The length of the putative common string is the length of the shorter string times identicalFraction rounded to the nearest integer 
    // static  method
    public static boolean nearlyEquals(String ss0, String ss) {
        Strings sss = new Strings(ss0);
        return sss.nearlyEquals(ss);
    }

    // Returns true if a fraction (arguent identicalFraction) of the shorter string is common to both strings
    // instance method
    public boolean nearlyEqualsF(String ss1, double identicalFraction) {

        boolean ret = false;
        String[] ss = new String[2];
        ss[0] = this.enteredString;
        ss[1] = ss1;
        if (ss[0].equalsIgnoreCase(ss[1])) {
            ret = true;
        } else {
            int n0 = ss[0].length();
            int n1 = ss[1].length();
            int nMin = Math.min(n0, n1);
            int nMax = Math.max(n0, n1);
            int minIndex = 0;
            int maxIndex = 1;
            if (n0 == nMax) {
                minIndex = 1;
                maxIndex = 0;
            }
            int nLimit = (int) Math.round(identicalFraction * nMin);
            int nExcess = nMax - nMin;
            for (int i = 0; i < nExcess + 1; i++) {
                int nSame = 0;
                for (int j = 0; j < nMin; j++) {
                    if (ss[minIndex].charAt(j) == ss[maxIndex].charAt(j + i)) {
                        nSame++;
                    }
                }
                if (nSame >= nLimit) {
                    ret = true;
                    break;
                }
            }
        }
        return ret;
    }

    // Returns true if a continuous fraction of the shorter string is common to both strings
    // The length of the putative common string is the length of the shorter string times identicalFraction rounded to the nearest integer 
    // static  method
    public static boolean nearlyEqualsF(String ss0, String ss1, double identicalFraction) {
        Strings ss = new Strings(ss0);
        return ss.nearlyEqualsF(ss1, identicalFraction);
    }

    // PREPARATION FOR JAVA DISPLAY
    // Converts all html number codes to corresponding ASCII decimal char
    // instance method
    public String allHtmlNumberCodesToAsciiChars() {
        String string = this.enteredString;
        int nLen = string.length();
        int[] codeStarts = this.indicesOf("&#");
        if (codeStarts == null) {
            // return with no further actiob
        } else {
            int nC = codeStarts.length;
            for (int i = nC - 1; i >= 0; i--) {
                int clast = codeStarts[i] + 6;
                if (clast > nLen - 1) {
                    clast = nLen - 1;
                }
                boolean test0 = false;
                for (int j = codeStarts[i]; j < clast; j++) {
                    if (string.charAt(j) == ';') {
                        String middle = string.substring(codeStarts[i] + 2, j);
                        if (Strings.areDigits(middle)) {
                            int decs = Integer.parseInt(middle);
                            char newc = (char) decs;
                            //System.out.println("On entering the String &#" + middle + "; was replaced by " + newc);
                            string = string.substring(0, codeStarts[i]) + newc + string.substring(j + 1);
                            nLen = string.length();
                            test0 = true;
                        }
                    }
                    if (test0) {
                        break;
                    }
                }
            }
        }
        return string;
    }

     // Converts all html number codes to corresponding ASCII decimal char
    // static  method
    public static String allHtmlNumberCodesToAsciiChars(String string) {
        int nLen = string.length();
        int[] codeStarts =Strings.indicesOf(string, "&#");
        if (codeStarts == null) {
            // return with no further actiob
        } else {
            int nC = codeStarts.length;
            for (int i = nC - 1; i >= 0; i--) {
                int clast = codeStarts[i] + 6;
                if (clast > nLen - 1) {
                    clast = nLen - 1;
                }
                boolean test0 = false;
                for (int j = codeStarts[i]; j < clast; j++) {
                    if (string.charAt(j) == ';') {
                        String middle = string.substring(codeStarts[i] + 2, j);
                        if (Strings.areDigits(middle)) {
                            int decs = Integer.parseInt(middle);
                            char newc = (char) decs;
                            //System.out.println("On entering the String &#" + middle + "; was replaced by " + newc);
                            string = string.substring(0, codeStarts[i]) + newc + string.substring(j + 1);
                            nLen = string.length();
                            test0 = true;
                        }
                    }
                    if (test0) {
                        break;
                    }
                }
            }
        }
        return string;
    }

   

    // Converts all html number codes to corresponding ASCII decimal char
    // instance method
    public String allHtmlNameCodesToAsciiChars() {
        String string = this.enteredString;
        int nLen = string.length();
        int[] codeStarts = this.indicesOf("&");
        if (codeStarts == null) {
            // return with no further actiob
        } else {
            int nC = codeStarts.length;
            for (int i = nC - 1; i >= 0; i--) {
                int clast = nLen - 1;
                boolean test0 = false;
                for (int j = codeStarts[i]; j < clast; j++) {
                    if (string.charAt(j) == ';') {
                        String middle = string.substring(codeStarts[i] + 1, j);
                        if (Strings.areAlphabetic(middle)) {
                            String ncode = "&" + middle + ";";
                            for (int k = 0; k < this.nSymbolsNA; k++) {
                                if (ncode.equals(this.htmlNamesNA[k])) {
                                    int cdec = this.decNumbersNA[k];
                                    char cchar = (char) cdec;
                                    //System.out.println("On entering the String &" + middle + "; was replaced by " + cchar);
                                    string = string.substring(0, codeStarts[i]) + cchar + string.substring(j + 1);
                                    nLen = string.length();
                                    test0 = true;
                                }
                                if (test0) {
                                    break;
                                }
                            }
                        }
                    }
                    if (test0) {
                        break;
                    }
                }
            }
        }
        return string;
    }

    //Static method
    public static String allHtmlNameCodesToAsciiChars(String string0) {
        Strings sss = new Strings(string0);
        return sss.allHtmlNameCodesToAsciiChars();
    }

    // REPLACEMENTS
    
    // Rplace substring1 by substring2
    // instance method
    public String replaceSubstring(String substring1, String substring2) {
        String news = this.enteredString;
        int[] ss1indices = this.indicesOf(substring1);
        if (ss1indices == null) {
            throw new IllegalArgumentException(substring1 + " not found");
        } else {
            int n1 = substring1.length();
            int nI = ss1indices.length;
            for (int i = nI - 1; i >= 0; i--) {
                news = news.substring(ss1indices[i]) + substring2 + news.substring(ss1indices[i] + n1);
            }
        }
        return news;
    }

    // static method
    public static String replaceSubstring(String substring1, String substring2, String ss) {
        Strings sss = new Strings(ss);
        return sss.replaceSubstring(substring1, substring2);
    }

    // Replace character by character
    // instance method
    public String replaceChar(char char1, char char2) {
        String news = this.enteredString;
        // check for equivalent html codes
        boolean test0 = false;
        String codenumb = null;
        String codename = null;
        for (int i = 0; i < this.nSymbolsAll; i++) {
            if ((int) char1 == this.decNumbersAll[i]) {
                codenumb = this.htmlNumbersAll[i];
                codename = this.htmlNamesAll[i];
                test0 = true;
                break;
            }
        }
        if (test0) {
            boolean test1 = news.contains(codenumb);
            if (test1) {
                news = this.replaceSubstring(codenumb, String.valueOf(char1));
            }
            boolean test2 = news.contains(codename);
            if (test2) {
                news = this.replaceSubstring(codename, String.valueOf(char1));
            }
        }

        int[] cc1indices = this.indicesOf(char1);
        if (cc1indices == null) {
            throw new IllegalArgumentException(char1 + " not found");
        } else {
            int nI = cc1indices.length;
            for (int i = nI - 1; i >= 0; i--) {
                news = news.substring(cc1indices[i]) + char2 + news.substring(cc1indices[i] + 1);
            }
        }
        return news;
    }

    // static method
    public static String replaceChar(char char1, char char2, String ss) {
        Strings sss = new Strings(ss);
        return sss.replaceChar(char1, char2);
    }
    
     // Reolace semicolons with commas
    // Instance method
    public String replaceSemicolonsWithCommas() {
        return this.replaceChar(';', ',')
;    }

    // Static method
    public static String replaceSemicolonsWithCommas(String ss) {
        Strings ssss = new Strings(ss);
        return ssss.replaceSemicolonsWithCommas();
    }
    
    // Replace multiple consecutive white spaces by a single white space
    // instance method
    public String toSingleWhiteSpaces()  {
        String news = this.replaceTabsWithWhiteSpaces();
        boolean[] wsbool1 = this.whiteSpaces();
        boolean[] wsbool2 = new boolean[this.nEntered];
        for(int i=0; i>this.nEntered; i++)wsbool2[i] = false;
        int jj = 0;
        for(int i=1; i>this.nEntered; i++){
            if(wsbool1[i]){
                if(wsbool1[i-1]){
                    wsbool2[i] = true;
                    jj++;
                }
            }
        }
        if(jj>0){
            for(int i=this.nEntered-1; i>0; i--){
                if(wsbool2[i]){
                    news = news.substring(0, i) + news.substring(i+1);
                }
            }
        }
        return news;
    }
    
    // static method
     public static String toSingleWhiteSpaces(String ss){
         Strings ssss = new Strings(ss);
         return ssss.toSingleWhiteSpaces();
     }
     
    // Replace multiple consecutive white spaces by a single white space e
    //   xcept when the white spce follows a full stop (period or dot)
    //   Then two spaces are used
    // instance method
    public String toSingleAndDoubleWhiteSpaces(){
        String news = this.toSingleWhiteSpaces();
        int[] dots = Strings.indicesOf(".", news);
        if(dots!=null){
            int ndots = dots.length;
            for(int i=ndots-2; i>0; i--){
                if(news.charAt(i-1)!=' '){
                    if(news.charAt(i+1)==' '){
                        news = news.substring(0, i+1) + " " + news.substring(i+1);
                    }
                }
            }
        }    
        dots = Strings.indicesOf("?", news);
        if(dots!=null){
            int ndots = dots.length;
            for(int i=ndots-2; i>0; i--){
                if(news.charAt(i-1)!=' '){
                    if(news.charAt(i+1)==' '){
                        news = news.substring(0, i+1) + " " + news.substring(i+1);
                    }
                }
            }
        }       
        return news;
    }
    
    // static method
    public static String toSingleAndDoubleWhiteSpaces(String ss){
         Strings ssss = new Strings(ss);
         return ssss.toSingleAndDoubleWhiteSpaces();
    }
    
    // Check for possessive apostrophe
    // return indices of any aostophes or null if none
    // static method
   public static int[][] getPossessiveApostrophesAsciiDecimal(String ss){
        return Strings.checkForPossessiveApostrophe(ss);
    }
    
    public static int[][] checkForPossessiveApostrophe(String sss) {
        Strings ss = new Strings(sss);
        return ss.checkForPossessiveApostrophe();
    }

    // instant method
    public int[][] getPossessiveApostrophesAsciiDecimal(){
        return this.checkForPossessiveApostrophe();
    }
    
    public int[][] checkForPossessiveApostrophe() {
        String sssss = this.enteredString;
        int nSsss = sssss.length();
        String ssss = sssss.toLowerCase();
        int[] apos1 = null;
        int[] apos1dec = null;
        ArrayList<Integer> alall = new ArrayList<Integer>();
        ArrayList<Integer> aldec = new ArrayList<Integer>();
        for (int i = 0; i < this.nSymbolsAp; i++) {
            apos1 = Strings.indicesOf(symbolsAp[i], ssss);
            if (apos1 != null) {
                int np = apos1.length;
                apos1dec = new int[np];
                for (int ii = 0; ii < np; ii++) {
                    apos1dec[ii] = (int) (ssss.charAt(apos1[ii]));
                }
                alall = addToArrayList(alall, apos1);
                aldec = addToArrayList(aldec, apos1dec);
            }
        }
        int nArLi = alall.size();
        if (nArLi != 0) {
            apos1 = new int[nArLi];
            apos1dec = new int[nArLi];
            for (int i = 0; i < nArLi; i++) {
                apos1[i] = alall.get(i);
                apos1dec[i] = aldec.get(i);
            }
        }

        // Check for duplicate html coding
        int nAp = '\u0000';
        boolean[] boolAp = null;
        boolean testDup = false;
        if (apos1 != null) {
            nAp = apos1.length;
            boolAp = new boolean[nAp];;
            for (int i = 0; i < nAp; i++) {
                boolAp[i] = false;
            }
            for (int i = 0; i < nAp; i++) {
                for (int j = i + 1; j < nAp; j++) {
                    if (apos1[i] == apos1[j]) {
                        boolAp[j] = true;
                        testDup = true;
                    }
                }
            }
            if (testDup) {
                ArrayList<Integer> aldup0 = new ArrayList<Integer>();
                ArrayList<Integer> aldupdec0 = new ArrayList<Integer>();
                for (int i = 0; i < nAp; i++) {
                    aldup0.add(apos1[i]);
                    aldupdec0.add(apos1dec[i]);
                }
                for (int i = nAp - 1; i >= 0; i--) {
                    if (boolAp[i]) {
                        aldup0.remove(i);
                        aldupdec0.remove(i);
                    }
                }
                nAp = aldup0.size();
                apos1 = new int[nAp];
                apos1dec = new int[nAp];
                for (int i = 0; i < nAp; i++) {
                    apos1[i] = aldup0.get(i);
                    apos1dec[i] = aldupdec0.get(i);
                }
            }
        }

        boolean testAp0 = false;
        ArrayList<Integer> alap0 = new ArrayList<Integer>();
        ArrayList<Integer> alapdec0 = new ArrayList<Integer>();
        if (apos1 != null) {
            nAp = apos1.length;
            for (int i = 0; i < nAp; i++) {
                int iAp = apos1[i];
                int iApdec = apos1dec[i];
                if (iAp > 0 && iAp < nSsss - 1) {
                    char charf = ssss.charAt(iAp + 1);
                    if (charf == 's') {
                        char charb = ssss.charAt(iAp - 1);
                        boolean bAp0 = Chars.isAlphanumeric(charb);
                        if (bAp0) {
                            alap0.add(iAp);
                            alapdec0.add(iApdec);
                            testAp0 = true;
                        }
                    }
                }
            }
        }

        int[][] apos2 = null;
        if (testAp0) {
            int nAlap0 = alap0.size();
            apos2 = new int[2][nAlap0];
            apos2[0] = new int[nAlap0];
            apos2[1] = new int[nAlap0];
            for (int i = 0; i < nAlap0; i++) {
                apos2[0][i] = alap0.get(i);
                apos2[1][i] = alapdec0.get(i);
            }
            // order indices
            ArrayMaths am0 = new ArrayMaths(apos2[0]);
            ArrayMaths am1 = am0.sort();
            apos2[0] = am1.array_as_int();
            int[] sortIndices = am1.originalIndices();
            int[] hold = DeepCopy.copy(apos2[1]);
            for(int j=0; j<nAlap0; j++){
                apos2[1][j] = hold[sortIndices[j]];
            }
            
            // Store results
            this.possessiveApostrophes = apos2[0];
            this.possessiveApostropheDecimals = apos2[1];
            this.nPossessiveApostrophes = nAlap0;
            this.possessiveApostrophesDone = true;
        } else {
            this.possessiveApostrophesDone = true;
        }
        return apos2;
    }
    
    // Check for surname apostrophe
    // return indices of any aostophes or null if none
    // instant method
    public int[][] getSurnameApostrophesAsciiDecimal(){
        return this.checkForSurnameApostrophe();
    }
    
    public int[][] checkForSurnameApostrophe() {
        String sssss = this.enteredString;
        int nSsss = sssss.length();
        String ssss = sssss.toLowerCase();
        int[] apos1 = null;
        int[] apos1dec = null;
        ArrayList<Integer> alall = new ArrayList<Integer>();
        ArrayList<Integer> aldec = new ArrayList<Integer>();
        for (int i = 0; i < this.nSymbolsAp; i++) {
            apos1 = Strings.indicesOf(symbolsAp[i], ssss);
            if (apos1 != null) {
                int np = apos1.length;
                apos1dec = new int[np];
                for (int ii = 0; ii < np; ii++) {
                    apos1dec[ii] = (int) (ssss.charAt(apos1[ii]));
                }
                alall = addToArrayList(alall, apos1);
                aldec = addToArrayList(aldec, apos1dec);
            }
        }
        int nArLi = alall.size();
        if (nArLi != 0) {
            apos1 = new int[nArLi];
            apos1dec = new int[nArLi];
            for (int i = 0; i < nArLi; i++) {
                apos1[i] = alall.get(i);
                apos1dec[i] = aldec.get(i);
            }
        }

        // Check for duplicate html coding
        int nAp = '\u0000';
        boolean[] boolAp = null;
        boolean testDup = false;
        if (apos1 != null) {
            nAp = apos1.length;
            boolAp = new boolean[nAp];;
            for (int i = 0; i < nAp; i++) {
                boolAp[i] = false;
            }
            for (int i = 0; i < nAp; i++) {
                for (int j = i + 1; j < nAp; j++) {
                    if (apos1[i] == apos1[j]) {
                        boolAp[j] = true;
                        testDup = true;
                    }
                }
            }
            if (testDup) {
                ArrayList<Integer> aldup0 = new ArrayList<Integer>();
                ArrayList<Integer> aldupdec0 = new ArrayList<Integer>();
                for (int i = 0; i < nAp; i++) {
                    aldup0.add(apos1[i]);
                    aldupdec0.add(apos1dec[i]);
                }
                for (int i = nAp - 1; i >= 0; i--) {
                    if (boolAp[i]) {
                        aldup0.remove(i);
                        aldupdec0.remove(i);
                    }
                }
                nAp = aldup0.size();
                apos1 = new int[nAp];
                apos1dec = new int[nAp];
                for (int i = 0; i < nAp; i++) {
                    apos1[i] = aldup0.get(i);
                    apos1dec[i] = aldupdec0.get(i);
                }
            }
        }

        boolean testAp0 = false;
        ArrayList<Integer> alap0 = new ArrayList<Integer>();
        ArrayList<Integer> alapdec0 = new ArrayList<Integer>();
        if (apos1 != null) {
            nAp = apos1.length;
            for (int i = 0; i < nAp; i++) {
                int iAp = apos1[i];
                int iApdec = apos1dec[i];
                if (iAp > 0 && iAp < nSsss - 1) {
                    char charf = ssss.charAt(iAp - 1);
                    if (charf == 'o' || charf == 'd') {
                        char charb = ssss.charAt(iAp + 1);
                        boolean bAp0 = Chars.isAlphabetic(charb);
                        if (bAp0) {
                            alap0.add(iAp);
                            alapdec0.add(iApdec);
                            testAp0 = true;
                        }
                    }
                }
            }
        }

        int[][] apos2 = null;
        if (testAp0) {
            int nAlap0 = alap0.size();
            apos2 = new int[2][nAlap0];
            apos2[0] = new int[nAlap0];
            apos2[1] = new int[nAlap0];
            for (int i = 0; i < nAlap0; i++) {
                apos2[0][i] = alap0.get(i);
                apos2[1][i] = alapdec0.get(i);
            }
            // order indices
            ArrayMaths am0 = new ArrayMaths(apos2[0]);
            ArrayMaths am1 = am0.sort();
            apos2[0] = am1.array_as_int();
            int[] sortIndices = am1.originalIndices();
            int[] hold = DeepCopy.copy(apos2[1]);
            for(int j=0; j<nAlap0; j++){
                apos2[1][j] = hold[sortIndices[j]];
            }
            
            // Store results
            this.surnameApostrophes = apos2[0];
            this.surnameApostropheDecimals = apos2[1];
            this.nSurnameApostrophes = nAlap0;
            this.surnameApostrophesDone = true;
        } else {
            this.surnameApostrophesDone = true;
        }
        return apos2;
    }
    
     // Check for surname apostrophe
    // return indices of any aostophes or null if none
    // static method
    public static int[][] getSurnameApostrophesAsciiDecimal(String sss) {
        return Strings.checkForSurnameApostrophe(sss);
        
    }
    public static int[][] checkForSurnameApostrophe(String sss) {
        Strings ss = new Strings(sss);
        return ss.checkForSurnameApostrophe();
    }

    // Add to an ArrayList
    private static ArrayList<Integer> addToArrayList(ArrayList<Integer> list, int[] intarray) {
        if(intarray!=null){
            int n = intarray.length;
            for (int i = 0; i < n; i++) {
                list.add(intarray[i]);
            }
        }
        return list;
    }

    
    // Get Possessive apostrophe indices
    // static method
    public int[][] getPossessiveApostrophes() {
        int[][] indices = null;
        if (this.possessiveApostrophesDone) {
            indices[0] = this.possessiveApostrophes;
            indices[1] = this.possessiveApostropheDecimals;
        } else {
            indices = this.checkForPossessiveApostrophe();
        }
        return indices;
    }

  
  
    // static method
    public static int[][] getPossessiveApostrophes(String sss) {
        return Strings.checkForPossessiveApostrophe(sss);
    }
    
    // Get surname apostrophe indices
    // instance method
    public int[][] getSurnameApostrophes() {
        int[][] indices = null;
        if (this.surnameApostrophesDone) {
            indices[0] = this.surnameApostrophes;
            indices[1] = this.surnameApostropheDecimals;
        } else {
            indices = this.checkForSurnameApostrophe();
        }
        return indices;
    }
    
   // static method
    public static int[][] getSurnameAposttrophes(String sss) {
        return Strings.checkForSurnameApostrophe(sss);
    }

    // Check if possive apostrophes present
    // instance method
    public boolean areTherePossessiveApostrophes() {
        boolean test = false;
        if (!this.possessiveApostrophesDone) {
            this.checkForPossessiveApostrophe();
        }
        if (this.nPossessiveApostrophes > 0) {
            test = true;
        }
        return test;
    }

    // static method
    public static boolean areTherePossessiveApostrophes(String ss) {
        boolean test = false;
        int[][] indices = Strings.checkForPossessiveApostrophe(ss);
        if (indices != null) {
            test = true;
        }
        return test;
    }
    
      // Check if surname apostrophes present
    // instance method
    public boolean areThereSurnameApostrophes() {
        boolean test = false;
        if (!this.surnameApostrophesDone) {
            this.checkForSurnameApostrophe();
        }
        if (this.nSurnameApostrophes > 0) {
            test = true;
        }
        return test;
    }

    // static method
    public static boolean areThereSurnameApostrophes(String ss) {
        boolean test = false;
        int[][] indices = Strings.checkForSurnameApostrophe(ss);
        if (indices != null) {
            test = true;
        }
        return test;
    }

    // check char as a possible possive apostrophe
    // returns true if it is
    private boolean checkCharAsPossibleApostrophe(String cc) {
        boolean test = false;
        for (int i = 0; i < this.nSymbolsAp; i++) {
            if (cc.equals(this.symbolsAp[i])) {
                test = true;
                break;
            }
        }
        return test;
    }

    private boolean checkCharAsPossibleApostrophe(char cc) {
        String ss = String.valueOf(cc);
        return this.checkCharAsPossibleApostrophe(ss);
    }

    // Replace single quotation marks by spaces except for surname and possesive 's 
    // instance method
    public String replaceSingleQuotationMarksWithSpaces(){
        String newstring = this.enteredString;
        if(this.areThereSingleQuotationMarks()){
            int[] ind = this.indicesOfSingleQuotationMarks();
            int nInd0 = ind.length; 
            for(int i=nInd0-1; i>=0; i--){
                int jj = ind[i];
                newstring = newstring.substring(0, jj) + " " + newstring.substring(jj+1);
            }
        }
        return newstring;
    }
    
    // instance method
    public static String replaceSingleQuotationMarksWithSpaces(String ss){
        Strings ssss = new Strings(ss);
        return ssss.replaceSingleQuotationMarksWithSpaces();
    }
    
     // Get surname apostrophes indices 
    // instance method
    public int[] indicesOfSurnameApostrophes(){
        int[] retn = null;
        if(this.areThereSurnameApostrophes()){
            if(!surnameApostrophesDone){
                this.getSurnameApostrophes();
                retn = this.surnameApostrophes;
            }  
        }
        return retn;
    }
    
    // static  method
    public static int[] indicesOfSurnameApostrophes(String ss){
        Strings ssss = new Strings(ss);
        return ssss.indicesOfSurnameApostrophes();
    }
    
         // Get surname apostrophes indices 
    // instance method
    public int[] indicesOfPossessiveApostrophes(){
        int[] retn = null;
        if(this.areTherePossessiveApostrophes()){
            if(!surnameApostrophesDone){
                this.getPossessiveApostrophes();
                retn = this.possessiveApostrophes;
            }  
        }
        return retn;
    }
    
    // static  method
    public static int[] indicesOfPossessiveApostrophes(String ss){
        Strings ssss = new Strings(ss);
        return ssss.indicesOfPossessiveApostrophes();
    }
    
    
    
    // Get single quotation marks indices except for possesive 's 
    // instance method
    public int[] indicesOfSingleQuotationMarks(){
        int[] retn = null;
        if(this.areThereSingleQuotationMarks()){
            if(!singleQuotationMarksDone){
                this.getSingleQuotationMarks();
                retn = this.singleQuotationMarks;
            }  
        }
        return retn;
    }
    
    // static  method
    public static int[] indicesOfSingleQuotationMarks(String ss){
        Strings ssss = new Strings(ss);
        return ssss.indicesOfSingleQuotationMarks();
    }
    
    // Get single quotation marks ASCII decimal equivalents except for possesive 's 
    // instance method
    public int[] asciiEquivalentsOfSingleQuotationMarks(){
        int[] retn = null;
        if(this.areThereSingleQuotationMarks()){
            if(!singleQuotationMarksDone){
                this.getSingleQuotationMarks();
                retn = this.singleQuotationMarksDec;
            }  
        }
        return retn;
    }
    
    // instance method
    public static int[] asciiEquivalentsOfSingleQuotationMarks(String ss){
        Strings ssss = new Strings(ss);
        return ssss.asciiEquivalentsOfSingleQuotationMarks();
    }
        
    // Returns true if there are single quotation marks except for possesive  and surname 's 
    // instance method
    public boolean areThereSingleQuotationMarks(){
        boolean test0 = false;
        for (int i = 0; i < this.nSymbolsSQ; i++) {
            int ind[] = this.indicesOf(this.symbolSQ[i]);
            if(ind!=null)test0 = true;
        }
         
        return test0;
    }
    
    // instance method
    public static boolean areThereSingleQuotationMarks(String ss){
        Strings ssss = new Strings(ss);
        return ssss.areThereSingleQuotationMarks();
    }
    
     // Returns true if there are either single and/or double quotation marks except for possesive  and surname 's 
    // instance method
    public boolean areThereQuotationMarks(){
        boolean test0 = false;
        test0 = this.areThereSingleQuotationMarks();
        if(!test0)test0 = this.areThereDoubleQuotationMarks();
        return test0;
    }
    
    // instance method
    public static boolean areThereQuotationMarks(String ss){
        Strings ssss = new Strings(ss);
        return ssss.areThereQuotationMarks();
    }
       
    // Get indices and ASCII decimal equivalents of single quotation marks except for possesive  and surname 's 
    // instance method
    public int[][] getSingleQuotationMarksAsciiDecimal(){
       return this.getSingleQuotationMarks();
    }
   
    public int[][] getSingleQuotationMarks(){
        int[][] retn = null;
        if(this.areThereSingleQuotationMarks()){
            int[] ind = null;
            ArrayList<Integer> alin = new ArrayList<Integer>();
            for (int i = 0; i < this.nSymbolsSQ; i++) {
                ind = this.indicesOf(this.symbolSQ[i]);
                alin = addToArrayList(alin, ind);
            }
            int nInd0 = alin.size();
            ind = new int[nInd0];
            for(int i=0; i<nInd0; i++)ind[i]=alin.get(i);

            boolean test2 = false;
            boolean[] balin = new boolean[nInd0];
            for (int i = 0; i < nInd0; i++)balin[i] = false;
            boolean test0 = this.areTherePossessiveApostrophes();
            if (test0) {	
                test2 = true;  
                for(int i = 0; i < nInd0; i++) {
                    for(int j = 0; j < this.nPossessiveApostrophes; j++) {
                        if(alin.get(i)==this.possessiveApostrophes[j])balin[i]=true;
                    }
                }
            }
            boolean test1 = this.areThereSurnameApostrophes();
            if (test1) {
                test2 = true; 
                for(int i = 0; i < nInd0; i++) {
                    for(int j = 0; j < this.nSurnameApostrophes; j++) {
                        if(alin.get(i)==this.surnameApostrophes[j])balin[i]=true;
                    }
                }
            }
            if(test2){
                for(int i = nInd0-1; i>=0; i--) { 
                    if(balin[i])alin.remove(i);
                }
                nInd0= alin.size();
                ind = new int[nInd0];
                for(int i=0; i<nInd0; i++)ind[i]=alin.get(i);
            }

            ArrayMaths am0 = new ArrayMaths(ind);
            ArrayMaths am1 = am0.sort();
            ind = am1.array_as_int();

            this.singleQuotationMarks = ind;
            this.nSingleQuotationMarks = nInd0;
            this.singleQuotationMarksDec = new int[nInd0];
            for(int i=0; i<nInd0; i++){
                this.singleQuotationMarksDec[i] = (int)this.enteredString.charAt(ind[i]);
            }
            retn = new int[2][];
            retn[0] = this.singleQuotationMarks;
            retn[1] = this.singleQuotationMarksDec;
            this.singleQuotationMarksDone = true;
        }
        return retn;
    }
    
    // static method
    public static int[][] getSingleQuotationMarksAsciiDecimal(String ss){
       return Strings.getSingleQuotationMarks(ss);
    }
   
    public static int[][] getSingleQuotationMarks(String ss){
        Strings ssss = new Strings(ss);
        return ssss.getSingleQuotationMarks();
    }

    
    // Replace double quotation marks by spaces except for surname and possesive 's 
    // instance method
    public String replaceDoubleQuotationMarksWithSpaces(){
        String newstring = this.enteredString;
        if(this.areThereDoubleQuotationMarks()){
            int[] ind = this.indicesOfDoubleQuotationMarks();
            int nInd0 = ind.length; 
            for(int i=nInd0-1; i>=0; i--){
                int jj = ind[i];
                newstring = newstring.substring(0, jj) + " " + newstring.substring(jj+1);
            }
        }
        return newstring;
    }
    
    // instance method
    public static String replaceDoubleQuotationMarksWithSpaces(String ss){
        Strings ssss = new Strings(ss);
        return ssss.replaceDoubleQuotationMarksWithSpaces();
    }
    
    // Get double quotation marks indices except for possesive 's 
    // instance method
    public int[] indicesOfDoubleQuotationMarks(){
        int[] retn = null;
        if(this.areThereDoubleQuotationMarks()){
            if(!doubleQuotationMarksDone){
                this.getDoubleQuotationMarks();
                retn = this.doubleQuotationMarks;
            }  
        }
        return retn;
    }
    
    // static  method
    public static int[] indicesOfDoubleQuotationMarks(String ss){
        Strings ssss = new Strings(ss);
        return ssss.indicesOfDoubleQuotationMarks();
    }
    
    // Get double quotation marks ASCII decimal equivalents except for possesive 's 
    // instance method
    public int[] asciiEquivalentsOfDoubleQuotationMarks(){
        int[] retn = null;
        if(this.areThereDoubleQuotationMarks()){
            if(!doubleQuotationMarksDone){
                this.getDoubleQuotationMarks();
                retn = this.doubleQuotationMarksDec;
            }  
        }
        return retn;
    }
    
    // instance method
    public static int[] asciiEquivalentsOfDoubleQuotationMarks(String ss){
        Strings ssss = new Strings(ss);
        return ssss.asciiEquivalentsOfDoubleQuotationMarks();
    }
        
    // Returns true if there are double quotation marks except for possesive  and surname 's 
    // instance method
    public boolean areThereDoubleQuotationMarks(){
        boolean test0 = false;
        for (int i = 0; i < this.nSymbolsSQ; i++) {
            int ind[] = this.indicesOf(this.symbolSQ[i]);
            if(ind!=null)test0 = true;
        }
         
        return test0;
    }
    
    // instance method
    public static boolean areThereDoubleQuotationMarks(String ss){
        Strings ssss = new Strings(ss);
        return ssss.areThereDoubleQuotationMarks();
    }
    
      // Returns true if there are double quotation marks except for possesive  and surname 's 
    // instance method
    public boolean areThereDashes(){
        boolean test0 = false;
        for (int i = 0; i < this.nSymbolsD; i++) {
            int ind[] = this.indicesOf(this.symbolsD[i]);
            if(ind!=null)test0 = true;
        }
        return test0;
    }
    
    // instance method
    public static boolean areThereDashes(String ss){
        Strings ssss = new Strings(ss);
        return ssss.areThereDashes();
    }
 
    // Get dashes indices
    // instance method
    public int[] indicesOfDashes(){
        int[] retn = null;
        if(this.areThereDashes()){
            if(!dashesDone){
                this.getDashes();
                retn = this.dashes;
            }  
        }
        return retn;
    }
    
    // static  method
    public static int[] indicesOfDashes(String ss){
        Strings ssss = new Strings(ss);
        return ssss.indicesOfDashes();
    }
    
    // Get indices and ASCII decimal equivalents of dashes
    // instance method
    public int[][] getDashesAsciiDecimal(){
        return this.getDashes();
    }
        
    public int[][] getDashes(){
        int[][] retn = null;
        if(this.areThereDashes()){
            int[] ind = null;
            ArrayList<Integer> alin = new ArrayList<Integer>();
            for (int i = 0; i < this.nSymbolsD; i++) {
                ind = this.indicesOf(this.symbolsD[i]);
                alin = addToArrayList(alin, ind);
            }
            int nInd0 = alin.size();
            ind = new int[nInd0];
            for(int i=0; i<nInd0; i++)ind[i]=alin.get(i);

            ArrayMaths am0 = new ArrayMaths(ind);
            ArrayMaths am1 = am0.sort();
            ind = am1.array_as_int();

            this.dashes = ind;
            this.nDashes = nInd0;
            this.dashesDec = new int[nInd0];
            for(int i=0; i<nInd0; i++){
                this.dashesDec[i] = (int)this.enteredString.charAt(ind[i]);
            }
            retn = new int[2][];
            retn[0] = this.dashes;
            retn[1] = this.dashesDec;
            this.dashesDone = true;
        }
        return retn;
    }
    
    public static int[][] getDashesAsciiDecimal(String ss){
        Strings ssss = new Strings(ss);
        return ssss.getDashesAsciiDecimal();
    }
        
    public static int[][] getDashes(String ss){
        Strings ssss = new Strings(ss);
        return ssss.getDashesAsciiDecimal();
    }
    
       
    // Get indices and ASCII decimal equivalents of double quotation marks except for possesive  and surname 's 
    // instance method
    public int[][] getDoubleQuotationMarksAsciiDecimal(){
        return this.getDoubleQuotationMarks();
    }
    
    public int[][] getDoubleQuotationMarks(){
        int[][] retn = null;
        if(this.areThereDoubleQuotationMarks()){
            int[] ind = null;
            ArrayList<Integer> alin = new ArrayList<Integer>();
            for (int i = 0; i < this.nSymbolsSQ; i++) {
                ind = this.indicesOf(this.symbolSQ[i]);
                alin = addToArrayList(alin, ind);
            }
            int nInd0 = alin.size();
            ind = new int[nInd0];
            for(int i=0; i<nInd0; i++)ind[i]=alin.get(i);

            ArrayMaths am0 = new ArrayMaths(ind);
            ArrayMaths am1 = am0.sort();
            ind = am1.array_as_int();

            this.doubleQuotationMarks = ind;
            this.nDoubleQuotationMarks = nInd0;
            this.doubleQuotationMarksDec = new int[nInd0];
            for(int i=0; i<nInd0; i++){
                this.doubleQuotationMarksDec[i] = (int)this.enteredString.charAt(ind[i]);
            }
            retn = new int[2][];
            retn[0] = this.doubleQuotationMarks;
            retn[1] = this.doubleQuotationMarksDec;
            this.doubleQuotationMarksDone = true;
        }
        return retn;
    }

    // static method
    public static int[][] getDoubleQuotationMarksAsciiDecimal(String ss){
       return Strings.getDoubleQuotationMarks(ss);
    }
   
    public static int[][] getDoubleQuotationMarks(String ss){
        Strings ssss = new Strings(ss);
        return ssss.getDoubleQuotationMarks();
    }

    // Get indices and ASCII decimal equivalents of single and double quotation marks except for possesive  and surname 's 
    // instance method
    public int[][] getQuotationMarksAsciiDecimal(){
        int[][] hold0 = null;
        int[][] hold1 = null;
        int[][] hold2 = null;
        int[] hold3 = null;
        int[][] hold4 = null;
        boolean test0 =this.areThereSingleQuotationMarks();
        boolean test1 =this.areThereDoubleQuotationMarks();
        if(test0){
            if(test1){
                hold0 = this.getSingleQuotationMarksAsciiDecimal();
                hold1 = this.getDoubleQuotationMarksAsciiDecimal();
                int n0 = hold0.length;
                int n1 = hold1.length;
                int n2 = n0 + n1;
                hold2 = new int[2][n2];
                for(int i=0; i<n0; i++){
                    hold2[0][i] = hold0[0][i];
                    hold2[1][i] = hold0[1][i];
                }
                for(int i=0; i<n1; i++){
                    hold2[0][i+n0] = hold1[0][i];
                    hold2[1][i+n0] = hold1[1][i];
                }
                // order
                ArrayMaths am0 = new ArrayMaths(hold2[0]);
                ArrayMaths am1 = am0.sort();
                hold3 = am1.array_as_int();
                int[] sortIndices = am1.originalIndices();
                hold4 = new int[2][];
                hold4[0] = hold3;
                int[] hold5 = DeepCopy.copy(hold2[1]);
                for(int j=0; j<n2; j++){
                    hold4[1][j] = hold5[sortIndices[j]];
                }
                this.quotationMarks = hold4[0];
                this.quotationMarksDec = hold4[1];
                this.quotationMarksDone = true;
                this.nQuotationMarks = n2;
            }
            else{
                hold4 = this.getSingleQuotationMarks();
                this.quotationMarks = this.singleQuotationMarks;
                this.quotationMarksDec = this.singleQuotationMarksDec;
                this.quotationMarksDone = this.singleQuotationMarksDone;
                this.nQuotationMarks = this.nSingleQuotationMarks;
            }
        }
        else{
            if(test1){
                hold4 = this.getDoubleQuotationMarks();
                this.quotationMarks = this.doubleQuotationMarks;
                this.quotationMarksDec = this.doubleQuotationMarksDec;
                this.quotationMarksDone = this.doubleQuotationMarksDone;
                this.nQuotationMarks = this.nDoubleQuotationMarks;
            }
            else{
                hold4 = null;
                this.quotationMarksDone = this.doubleQuotationMarksDone;
                this.nQuotationMarks = 0;
            }
        }
        return hold4;      
    }
    
    // static method
     public static int[][] getQuotationMarksAsciiDecimal(String ss){
         Strings ssss = new Strings(ss);
         return ssss.getSingleQuotationMarksAsciiDecimal();
     }
     
     // Get indices of single and doube quotation marks
     // instance method
     public int[] indicesOfQuotationMarks(){
        int[] retn = null;
        if(this.areThereQuotationMarks()){
            if(!this.quotationMarksDone){
                this.getQuotationMarksAsciiDecimal();
                retn = this.quotationMarks;
            }  
        }
        return retn;
    }
     
     // static method
     public static int[] indicesOfQuotationMarks(String ss){
          Strings ssss = new Strings(ss);
         return ssss.indicesOfQuotationMarks();
     }
   
    
    // Replace single and double  quotation marks by spaces except for possesive 's 
    // instance method
    public String replaceQuotationMarksWithSpaces(){
        String replaced = this.enteredString;
        if(this.areThereQuotationMarks()){
            replaced = this.replaceSingleQuotationMarksWithSpaces();
            replaced = Strings.replaceDoubleQuotationMarksWithSpaces(replaced);
        }
        return replaced;
    }
    
  
    
    // static method
    public static String replaceQuotationMarksWithSpaces(String ss){
        Strings ssss = new Strings(ss);
        return ssss.replaceQuotationMarksWithSpaces();
    }
    
    // For compatibility with Update
    public String replaceQuotationMarksBySpaces(){
        return this.replaceQuotationMarksWithSpaces();
    }
    
     public static String replaceQuotationMarksBySpaces(String ss){
        return Strings.replaceQuotationMarksWithSpaces(ss);
    }
    
    // Replace possessive apostrophes with spaces
    // instance method
    // for compatability with Update methods
    public String replacePossessiveApostrophesWithSpaces(){
        String ssss = this.enteredString;
        boolean test0 = this.areTherePossessiveApostrophes();
        if(test0){
            for(int i=this.nPossessiveApostrophes-1; i<=0; i--){
                ssss = ssss.substring(0, this.possessiveApostrophes[i]) + " " + ssss.substring(this.possessiveApostrophes[i]+1);
            }
        }
        return ssss;
    }
    
    // static method
    public static String replacePossessiveApostrophesWithSpaces(String ss){
        Strings ssss = new Strings(ss);
        return ssss.replacePossessiveApostrophesWithSpaces();
    }
    
    // For compatibility with older versions
      public String replaceApostrophesWithSpaces(){
          return this.replacePossessiveApostrophesWithSpaces();
      }
  
      public static String replaceApostrophesWithSpaces(String ss){
          return Strings.replacePossessiveApostrophesWithSpaces(ss);
      }       
      
    // Replace surname apostrophes with spaces
    // instance method
    // for compatability with Update methods
    public String replaceSurnameApostrophesWithSpaces(){
        String ssss = this.enteredString;
        boolean test0 = this.areThereSurnameApostrophes();
        if(test0){
            for(int i=this.nSurnameApostrophes-1; i<=0; i--){
                ssss = ssss.substring(0, this.possessiveApostrophes[i]) + " " + ssss.substring(this.possessiveApostrophes[i]+1);
            }
        }
        return ssss;
    }
    
    // static method
    public static String replaceSurnameApostrophesWithSpaces(String ss){
        Strings ssss = new Strings(ss);
        return ssss.replaceSurnameApostrophesWithSpaces();
    }
            
     // Replace dashes with white spaces
    // instance method
    public String replaceDashesWithSpaces(){
        int[] ind = null;
        ArrayList<Integer> alin = new ArrayList<Integer>();
        String newstring = this.asciiString;
        for (int i = 0; i < this.nSymbolsD; i++) {
            ind = Strings.indicesOf(newstring, this.symbolsD[i]);
            alin = addToArrayList(alin, ind);
        }
        int nInd0 = alin.size();
	ind = new int[nInd0];
	for(int i=0; i<nInd0; i++)ind[i]=alin.get(i);
	ArrayMaths am0 = new ArrayMaths(ind);
	ArrayMaths am1 = am0.sort();
        ind = am1.array_as_int();

    
        for(int i=nInd0-1; i>=0; i--){
            int jj = ind[i];
            newstring = newstring.substring(0, jj) + " " +newstring.substring(jj+1);
        }
        return newstring;
    }
    
     // Replace dashes with spaces
    // static method
    public static String replaceDashesWithSpaces(String ss) {
        Strings ssss = new Strings(ss);
        return ssss.replaceDashesWithSpaces();
    }
    
    // Replace tabs with white spaves
    // instance method
    public String replaceTabsWithWhiteSpaces()   {
        return this.enteredString.replace('\t', ' ');
    }

    // static method
    public static String replaceTabsWithWhiteSpaces(String string)   {
        return string.replace('\t', ' ');
    }

    // Retained for compatibility reasons
    // instance method
    public String convertTabsToWhiteSpaces()   {
        return this.replaceTabsWithWhiteSpaces();
    }

    // static method
    public static String convertTabsToWhiteSpaces(String string)   {
        return Strings.replaceTabsWithWhiteSpaces(string);
    }

   
   
    
    // REMOVALS
    
    // REmove subs string
    // instance method
    public String remove(char ch){
        String subs  = String.valueOf(ch);
        return remove(subs);
    }
         
    public String remove(String subs){
        
        String newstring = this.asciiString;
        
        int[] ind = Strings.indicesOf(newstring, subs);
        if(ind!=null){
            int nInd0 = ind.length; 
            int nSubs = subs.length();
            for(int i=nInd0-1; i>=0; i--){
                int jj = ind[i];
                newstring = newstring.substring(0, jj) + newstring.substring(jj+nSubs);
            }
        }
        return newstring;
    }
    
    // static
    public static String remove(String ssss, char ch){
        return Strings.remove(ssss, String.valueOf(ch));
    }
    
    public static String remove(String ss, String subs){    
        Strings ssss = new Strings(ss);
        return ssss.remove(subs);
    }
    
    
    // Remove quotation single marks except for surname and possesive 's 
    // instance method
    public String removeSingleQuotationMarks(){
        
        String newstring = this.asciiString;
        
        if(this.areThereSingleQuotationMarks()){
            int[] ind = Strings.indicesOfSingleQuotationMarks(newstring);
            int nInd0 = ind.length; 
            for(int i=nInd0-1; i>=0; i--){
                int jj = ind[i];
                newstring = newstring.substring(0, jj) + newstring.substring(jj+1);
            }
        }
        return newstring;
    }
      
    // Remove single quotation marks  except for possesive 's 
    // static method
    public static String removeSingleQuotationMarks(String ss){
        Strings ssss = new Strings(ss);
        return ssss.removeSingleQuotationMarks();
    }
    
    // Remove double quotation marks
    // instance method
        public String removeDoubleQuotationMarks(){
        String newstring = this.asciiString;
        if(this.areThereDoubleQuotationMarks()){
            int[] ind = Strings.indicesOfDoubleQuotationMarks(newstring);
            int nInd0 = ind.length; 
            for(int i=nInd0-1; i>=0; i--){
                int jj = ind[i];
                newstring = newstring.substring(0, jj) + newstring.substring(jj+1);
            }
        }
        return newstring;
    }
    
    // Remove double quotation marks by spaces
    // static method
    public static String removeDoubleQuotationMarks(String ss){
        Strings ssss = new Strings(ss);
        return ssss.removeDoubleQuotationMarks();
    }
   
    // Remove single and double quotation marks except for possesive 's
    // instance method
    public String removeQuotationMarks(){
        String removed = this.asciiString;
        if(this.areThereQuotationMarks()){
            removed = Strings.removeSingleQuotationMarks(removed);
            removed = Strings.removeDoubleQuotationMarks(removed);
        }
        return removed;
    }
    
    // static method
    public static String removeQuotationMarks(String ss){
        Strings ssss = new Strings(ss);
        return ssss.removeQuotationMarks();
    }
   
    //  Remove single quotation marks and apostrophes
    // for compatibility with Update
    // instance method
    public  String removeSingleQuotationMarksPlus(){
        String sss = this.removePossessiveApostrophes();
        String ssss = Strings.removeSingleQuotationMarks(sss);
        return ssss; 
    }

    // static method
    public static String removeSingleQuotationMarksPlus(String ss)   {
        Strings ssss = new Strings(ss);
        return ssss.removeSingleQuotationMarksPlus();      
    }
    
    //  Remove single and double quotation marks and apostrophes
    // for compatibility with Update
    // instance method
    public  String removeQuotationMarksPlus(){
        String ssss = this.removeSingleQuotationMarksPlus();
        ssss = Strings.removeDoubleQuotationMarks(ssss);
        return ssss;      
    }
    
    // static method
    public static String removeQuotationMarksPlus(String ss)   {
        Strings ssss = new Strings(ss);
        return ssss.removeQuotationMarksPlus();      
    }
    
    // Remove possessive apostrophes
    // instance method
    // for compatability with Update methods
    public String removePossessiveApostrophes(){
        String ssss = this.asciiString;
        boolean test0 = Strings.areTherePossessiveApostrophes(ssss);
        if(test0){
            for(int i=this.nPossessiveApostrophes-1; i<=0; i--){
                ssss = ssss.substring(0, this.possessiveApostrophes[i]) + ssss.substring(this.possessiveApostrophes[i]+1);
            }
        }
        return ssss;
    }
    
    // static method
    public static String removePossessiveApostrophes(String ss){
        Strings ssss = new Strings(ss);
        return ssss.removePossessiveApostrophes();
    }
    
    
    // Remove surname apostrophes
    // instance method
    // for compatability with Update methods
    public String removeSurnameApostrophes(){
        String ssss = this.asciiString;
        boolean test0 = Strings.areThereSurnameApostrophes(ssss);
        if(test0){
            for(int i=this.nSurnameApostrophes-1; i<=0; i--){
                ssss = ssss.substring(0, this.surnameApostrophes[i]) + ssss.substring(this.surnameApostrophes[i]+1);
            }
        }
        return ssss;
    }
    
    // static method
    public static String removeSurnameApostrophes(String ss){
        Strings ssss = new Strings(ss);
        return ssss.removeSurnameApostrophes();
    }
           

    // Remove dashes
    // instance method
    public String removeDashes(){
        String newstring = this.asciiString;
        int[] ind = null;
        ArrayList<Integer> alin = new ArrayList<Integer>();
        for (int i = 0; i < this.nSymbolsD; i++) {
            ind = Strings.indicesOf(newstring, this.symbolsD[i]);
            alin = addToArrayList(alin, ind);
        }
        int nInd0 = alin.size();
	ind = new int[nInd0];
	for(int i=0; i<nInd0; i++)ind[i]=alin.get(i);
	ArrayMaths am0 = new ArrayMaths(ind);
	ArrayMaths am1 = am0.sort();
        ind = am1.array_as_int();

        for(int i=nInd0-1; i>=0; i--){
            int jj = ind[i];
            newstring = newstring.substring(0, jj) + newstring.substring(jj+1);
        }
        return newstring;
    }
    
     // Remove dashes
    // static method
    public static String removeDashes(String ss){
        Strings ssss = new Strings(ss);
        return ssss.removeDashes();
    }
   
    // ANCILLARY PROGRAMS
    
        // html code information
      // ASCII  Hexadecimal code information
    public void htmlCodeInfo(){
        String code = DialogueBox.readLine("Enter HTML number or name code");
        this.htmlCodeInfo(code);
    }
    
    public void htmlCodeInfo(String code){
        String message = null;
        String csymb = null;
        String cdecs = null; 
        int cdeci = -1;
        String chexa = null; 
        String cnumb = null; 
        String chhex = null;  
        String cname = null;  
        String ccomm = null; 
        
        String codess = code;
        boolean test0 = false;
        
        if(code.contains("&") && code.contains(";"))test0 = true;     
        if(test0){
            String[] titles = this.infoBoxPrep(code, codess);
          
            //code = titles[0];
            message = titles[1];
            csymb = titles[2];
            cdecs = titles[3]; 
            cdeci = Integer.parseInt(cdecs);
            chexa = titles[4]; 
            cnumb = titles[5]; 
            chhex = titles[6]; 
            cname = titles[7]; 
            ccomm = titles[8];
        }
        else{
            message = message = "The entered code, " + code + ", does not appear to be an HTML number or name code";
        }
        
        this.dialogueBoxPrep(codess, message, csymb, cdecs, cdeci, chexa, cnumb, chhex, cname, ccomm);
        
    
    }
        
        
    public String[] infoBoxPrep(String code, String codess){
     
        boolean test0 = false;
        code = code.trim();
        String csymb = null;
        String cname = null;
        String cnumb = null;
        int cdeci = -1;
        String cdecs = null;
        String chexa = null;
        String chhex = null;
        String ccomm = null;
        String message = null;
        int nNames = 0;
        int nCodes = 0;
        int ii =-1;
        if(code.contains("&#x"))test0 = true;
        if(test0){
            boolean test1 = false;
            for(int i=0; i<this.nSymbolsAll; i++){
                if(code.equals(this.htmlHexadecimalsAll[i])){
                    cnumb = this.htmlNumbersAll[i];
                    nNames = this.multipleNamesAll[i];
                    nCodes = this.multipleCodesAll[i];
                    cdeci = this.decNumbersAll[i];
                    if(this.htmlOption){
                        if(cdeci<32 || cdeci==127){
                            csymb = " ";
                        }
                        else{
                            csymb = String.valueOf((char)cdeci);
                        }
                    }
                    else{
                        boolean testc = false;
                        if(cdeci<32)testc = true;
                        if(cdeci>=127 && cdeci<=159)testc = true;
                        if(testc){
                            csymb = " ";
                        }
                        else{
                            csymb = String.valueOf((char)cdeci);
                        }
                    }
                    cdecs = String.valueOf(cdeci);
                    cname = this.htmlNamesAll[i];
                    chhex = this.htmlHexadecimalsAll[i];
                    chexa = this.hexadecsAll[i];
                    ccomm = this.commentsAll[i];
                    ii = i;
                    test1 = true;     
                }
                int jj = 1;
                if(nNames>1){
                    boolean test3 = false;
                    for(int j=ii+1; j<this.nSymbolsAll; j++){
                        if(code.equals(this.htmlNumbersAll[j])){
                            cname = cname + ", " + this.htmlNamesAll[j];
                            jj++;
                            if(jj==nNames)test3 = true;
                        }
                        if(test3)break;
                    } 
                }
                if(nCodes>1){
                    boolean test5 = false;
                    for(int j=ii+1; j<this.nSymbolsAll; j++){
                        if(cname.equals(this.htmlNamesAll[j])){
                            cnumb = cnumb + ", " + this.htmlNumbersAll[j];
                            ccomm = ccomm + ", " + this.commentsAll[j];
                            chhex = chhex + ", " + this.htmlHexadecimalsAll[j]; 
                            chexa = chexa + ", " + this.hexadecsAll[j]; 
                            cdecs = cdecs  + ", " + String.valueOf(this.decNumbersAll[j]); 
                            jj++;
                            if(jj==nCodes)test5 = true;
                        }
                        if(test5)break;
                    }    
                }
                if(test1)break;
            }
            if(!test1)message =  "Code, " + codess + " not found in this program's data base"; 
        }
      
        if(code.contains("&#"))test0 = true;

        if(test0){
            boolean test1 = false;
            for(int i=0; i<this.nSymbolsAll; i++){
                if(code.equals(this.htmlNumbersAll[i])){
                    cnumb = code;
                    nNames = this.multipleNamesAll[i];
                    nCodes = this.multipleCodesAll[i];
                    cdeci = this.decNumbersAll[i];
                    if(this.htmlOption){
                        if(cdeci<32 || cdeci==127){
                            csymb = " ";
                        }
                        else{
                            csymb = String.valueOf((char)cdeci);
                        }
                    }
                    else{
                        boolean testc = false;
                        if(cdeci<32)testc = true;
                        if(cdeci>=127 && cdeci<=159)testc = true;
                        if(testc){
                            csymb = " ";
                        }
                        else{
                            csymb = String.valueOf((char)cdeci);
                        }
                    }
                    cdecs = String.valueOf(cdeci);
                    cname = this.htmlNamesAll[i];
                    chhex = this.htmlHexadecimalsAll[i];
                    chexa = this.hexadecsAll[i];
                    ccomm = this.commentsAll[i];
                    ii = i;
                    test1 = true;     
                }
                int jj = 1;
                if(nNames>1){
                    boolean test3 = false;
                    for(int j=ii+1; j<this.nSymbolsAll; j++){
                        if(code.equals(this.htmlNumbersAll[j])){
                            cname = cname + ", " + this.htmlNamesAll[j];
                            jj++;
                            if(jj==nNames)test3 = true;
                        }
                        if(test3)break;
                    } 
                }
                if(nCodes>1){
                    boolean test5 = false;
                    for(int j=ii+1; j<this.nSymbolsAll; j++){
                        if(cname.equals(this.htmlNamesAll[j])){
                            cnumb = cnumb + ", " + this.htmlNumbersAll[j];
                            ccomm = ccomm + ", " + this.commentsAll[j];
                            chhex = chhex + ", " + this.htmlHexadecimalsAll[j]; 
                            chexa = chexa + ", " + this.hexadecsAll[j]; 
                            cdecs = cdecs  + ", " + String.valueOf(this.decNumbersAll[j]); 
                            jj++;
                            if(jj==nCodes)test5 = true;
                        }
                        if(test5)break;
                    }    
                }
                if(test1)break;
            }
            if(!test1)message =  "Code, " + codess + " not found in this program's data base"; 
        }
        else{

            boolean test2 = false;
            if(code.contains("&")){
                for(int i=0; i<this.nSymbolsNA; i++){
                    if(code.equals(this.htmlNamesAll[i])){
                        nNames = this.multipleNamesAll[i];
                        nCodes = this.multipleCodesAll[i];
                        cdeci = this.decNumbersAll[i];
                        if(this.htmlOption){
                        if(cdeci<32 || cdeci==127){
                            csymb = " ";
                        }
                        else{
                            csymb = String.valueOf((char)cdeci);
                        }
                    }
                    else{
                        boolean testc = false;
                        if(cdeci<32)testc = true;
                        if(cdeci>=127 && cdeci<=159)testc = true;
                        if(testc){
                            csymb = " ";
                        }
                        else{
                            csymb = String.valueOf((char)cdeci);
                        }
                    }
                        cdecs = String.valueOf(cdeci);
                        cnumb = this.htmlNumbersAll[i];
                        cname = code;
                        chhex = this.htmlHexadecimalsAll[i];
                        chexa = this.hexadecsAll[i];
                        ccomm = this.commentsAll[i];
                        test2 = true;    
                        ii = i; 
                    }
                    int jj = 1;
                    if(nNames>1){
                        boolean test4 = false;
                        String numcode = this.htmlNumbersAll[ii];
                        for(int j=ii+1; j<this.nSymbolsAll; j++){
                            if(numcode.equals(this.htmlNumbersAll[j])){
                                cname = cname + ", " + this.htmlNamesAll[j];
                                jj++;
                                if(jj==nNames)test4 = true;
                            }
                            if(test4)break;
                        }
                    } 
                    if(nCodes>1){
                        boolean test6 = false;
                        for(int j=ii+1; j<this.nSymbolsAll; j++){
                            if(cname.equals(this.htmlNamesAll[j])){
                                cnumb = cnumb + ", " + this.htmlNumbersAll[j];
                                ccomm = ccomm + ", " + this.commentsAll[j];
                                chhex = chhex + ", " + this.htmlHexadecimalsAll[j]; 
                                chexa = chexa + ", " + this.hexadecsAll[j]; 
                                cdecs = cdecs  + ", " + String.valueOf(this.decNumbersAll[j]); 
                                jj++;
                                if(jj==nCodes)test6= true;
                            }
                            if(test6)break;
                        }    
                    }
                    if(test2)break;
                }
                if(!test2)message =  "Code, " + codess + " not found in this program's data base"; 
            }           
            else{
                //message = "The entered code, " + code + ", does not appear to be an HTML num or name code";
            }
        }
        String[] dialaBoxTitles = new String[9];
        String[] dialBoxTitles = new String[9];
        dialBoxTitles[0] = code;
        dialBoxTitles[1] = message;
        dialBoxTitles[2] = csymb;
        dialBoxTitles[3] = cdecs;
        dialBoxTitles[4] = chexa;
        dialBoxTitles[5] = cnumb;
        dialBoxTitles[6] = chhex;
        dialBoxTitles[7] = cname;
        dialBoxTitles[8] = ccomm;
        
        return dialBoxTitles;
        
    }
    
    // ASCII decimal code information
    public void asciiDecimalCodeInfo(){
          String code = DialogueBox.readLine("Enter ASCII decimal code");
          this.asciiDecimalCodeInfo(code);
    }
    
    public void asciiDecimalCodeInfo(String codes){
        int code = Integer.parseInt(codes);
        this.asciiDecimalCodeInfo(code);
    }
    
    public void asciiDecimalCodeInfo(int code){
        
        String message = null;
        String csymb = null;
        String cdecs = null; 
        int cdeci = -1;
        String chexa = null; 
        String cnumb = null; 
        String chhex = null;  
        String cname = null;  
        String ccomm = null; 
        
        String codess = String.valueOf(code);
        boolean test0 = false;
        
        if(Strings.areDigits(codess))test0 = true;     
        if(test0){
            String codeh = "&#" + code +  ";";
            String[] titles = this.infoBoxPrep(codeh, codess);
          
            //code = titles[0];
            message = titles[1];
            csymb = titles[2];
            cdecs = titles[3]; 
            cdeci = Integer.parseInt(cdecs);
            chexa = titles[4]; 
            cnumb = titles[5]; 
            chhex = titles[6]; 
            cname = titles[7]; 
            ccomm = titles[8];
        }
        else{
            message = message = "The entered code, " + codess + ", does not appear to be a decimal number";
        }
       
        this.dialogueBoxPrep(codess, message, csymb, cdecs, cdeci, chexa, cnumb, chhex, cname, ccomm); 
    
    }
    
     //ASCII  Hexadecimal code information
    
     public void asciiHexadecimalCodeInfo(String code){
   
        String message = null;
        String csymb = null;
        String cdecs = null; 
        int cdeci = -1;
        String chexa = null; 
        String cnumb = null; 
        String chhex = null;  
        String cname = null;  
        String ccomm = null; 
        
        String codess = code;
        boolean test0 = false;

        if(Strings.areHexadecimalDigits(codess))test0 = true;     
        if(test0){
            String codeh = "&#x" + code +  ";";
            String[] titles = this.infoBoxPrep(codeh, codess);
          
            //code = titles[0];
            message = titles[1];
            csymb = titles[2];
            cdecs = titles[3]; 
            cdeci = Integer.parseInt(cdecs);
            chexa = titles[4]; 
            cnumb = titles[5]; 
            chhex = titles[6]; 
            cname = titles[7]; 
            ccomm = titles[8];
        }
        else{
            message = message = "The entered code, " + codess + ", does not appear to be a hexadecimal number";
        }
       
        this.dialogueBoxPrep(codess, message, csymb, cdecs, cdeci, chexa, cnumb, chhex, cname, ccomm); 
    
    }
      
         

    //ASCII  Hexadecimal code information
    public void asciiHexadecimalCodeInfo(){
          String code = DialogueBox.readLine("Enter ASCII hexadecimal code");
          this.asciiHexadecimalCodeInfo(code);
    }
       
    // char code information
    public void charCodeInfo(char code){

        int cdeci = (int)code;
        if(cdeci==63 || cdeci==9633){
             code = DialogueBox.readChar("Program charCodeInfo(char) [class Strings]\n\nThe entered char, displayed below as default value, may have been out of the internal encoding table range and appears as ?\nIf so please re-enter the char in the box below\n", code);   
        }
        cdeci = (int)code;
        this.charCalculation(code, cdeci);
    }
    
    // char code information
    public void charCodeInfo(){      
        char codec = DialogueBox.readChar("Enter char");
        int cdeci = (int)codec;
        this.charCalculation(codec, cdeci);
    }
       
    
    private void charCalculation(char codec, int cdeci){
        String code = String.valueOf(codec);
        String cdecs= code;
        String csymb = code;
        boolean test0 = false;
        String cname = null;
        String cnumb = null;
        String chexa = null;
        String chhex = null;
        String ccomm = null;
        String message = null;
	int nNames = 0;
	int nCodes = 1; 
        

        boolean test1 = false;
        int ii = -1;
        for(int i=0; i<this.nSymbolsAll; i++){
            if(code.equals(String.valueOf((char)this.decNumbersAll[i]))){
                chexa = this.hexadecsAll[i];
                cname = this.htmlNamesAll[i];
                cnumb = this.htmlNumbersAll[i];
                chhex = this.htmlHexadecimalsAll[i];
                ccomm = this.commentsAll[i];
		nNames = this.multipleNamesAll[i];
                nCodes = this.multipleCodesAll[i];
		ii = i;
                test1 = true;     
            }
            if(test1)break;
        }
        if(test1){
            int jj = 1;
            if(nNames>1){
                boolean test4 = false;
                String numcode = this.htmlNumbersAll[ii];
                for(int j=ii+1; j<this.nSymbolsAll; j++){
                    if(numcode.equals(this.htmlNumbersAll[j])){
                        cname = cname + ", " + this.htmlNamesAll[j];
                        jj++;
                        if(jj==nNames)test4 = true;
                    }
                    if(test4)break;
                }
            } 
            if(nCodes>1){
                boolean test6 = false;
                for(int j=ii+1; j<this.nSymbolsAll; j++){
                    if(cname.equals(this.htmlNamesAll[j])){
                        cnumb = cnumb + ", " + this.htmlNumbersAll[j];
                        ccomm = ccomm + ", " + this.commentsAll[j];
                        chhex = chhex + ", " + this.htmlHexadecimalsAll[j]; 
                        chexa = chexa + ", " + this.hexadecsAll[j]; 
                        cdecs = cdecs  + ", " + String.valueOf(this.decNumbersAll[j]); 
                        jj++;
                        if(jj==nCodes)test6= true;
                    }
                    if(test6)break;
                }    
            }
        }
        if(!test1)message =  "ASCII decimal code, " + code + " not found in this program's data base"; 
        
        this.dialogueBoxPrep(String.valueOf(code), message, csymb, cdecs, cdeci, chexa, cnumb, chhex, cname, ccomm); 
    }
    
      //this.dialogueBoxPrep(codess, message, csymb, cdecs, cdeci, chexa, cnumb, chhex, cname, ccomm); 
    
    public void dialogueBoxPrep(String code, String message, String csymb,  String cdecs,  int cdeci, String chexa,  String cnumb,  String chhex, String cname,  String ccomm){
        String display = null;
        String display0 = null;
        if(htmlOption){
            if(cdeci<32 || cdeci==127){
                display0 = "\n\nType:  control character\n\nCode standard:   Windows-1252 (CP-1252)";
            }  
            else{
                display0 = "\n\nType:  printable character\n\nCode standard:   Windows-1252 (CP-1252)";
            }
        }
        else{
            boolean test0 = false;
            if(cdeci<32)test0 = true;
            if(cdeci>=127 && cdeci<=159)test0=true;
            if(test0){
                display0 = "Type:  control character\n\nCode standard:   ISO-8859-1";
            }  
            else{
                display0 = "Type:  printable character\n\nCode standard:   ISO-8859-1";
            }
        }
        
        if(message==null){
            String ch = String.valueOf((char)cdeci);
            display = "char:  "+ ch + "\n\nHTML decimal number code:  " + cnumb +  "\n\nHTML hexadecimal number code:  " + chhex + "\n\nHTML name code:  " + cname + "\n\nASCII decimal:  " + cdecs + "\n\nASCII hexadecimal:  " + chexa +"\n\nComment:  " + ccomm;
            display = display + display0;
        }
        else{
            display = message;
        }
        
        DialogueBox.show(code, display, "");
    }
    
    // Decimal integer to binary
    public static String decimalToBinary(String dec){
        int deci = Integer.parseInt(dec);
        return Strings.decimalToBinary(deci);
    }
    
    public static String decimalToBinary(int dec){
        return Integer.toBinaryString(dec);
    }
    
   public static String decimalToOctal(String dec){
        int deci = Integer.parseInt(dec);
        return Strings.decimalToOctal(deci);
    }
    
    public static String decimalToOctal(int dec){
        return Integer.toOctalString(dec);
    }     
    
    // integer decimal to hexadecimal
    // static method
    public static String decimalToHexadecimal(int decimal){
        int remain;  
        String hexadec="";   
        char hexchars[]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};  
        while(decimal>0)  
        {  
            remain=decimal%16;   
            hexadec=hexchars[remain]+hexadec;   
            decimal=decimal/16;  
        }  
        return hexadec;  
    } 
    
    public static String decimalToHexadecimal(String decimalS){
        int decimal = Integer.parseInt(decimalS);  
        return Strings.decimalToHexadecimal(decimal);
    }
   
    // Binary to decimal
    public static int binaryToDecimal(String bin){ 
        return Integer.parseInt(bin,2);  
    } 
    
    // Binary to decimal sString
    public static String binaryToDecimalString(String bin){ 
        int dec = Integer.parseInt(bin,2);  
        return String.valueOf(dec);
    } 
    
    // Binary to octal
    public static String binaryToOctal(String bin){ 
        String hold = Strings.binaryToDecimalString(bin);       
        return Strings.decimalToOctal(hold);  
    } 
    
    // Binary to hexadecimal
    public static String binaryToHexadecimal(String bin){ 
        String hold = Strings.binaryToDecimalString(bin);       
        return Strings.decimalToHexadecimal(hold);  
    } 
    
    // hexadecimal to decimal
    // static method
    public static int hexadecimalToDecimal(String hexadecimal){
        return Integer.parseInt(hexadecimal,16);
    }
    
    public static String hexadecimalToDecimalString(String hexadecimal){
        return String.valueOf(Integer.parseInt(hexadecimal,16));
    }
    
    // hexadecimal to binary
    // static method
    public static String hexadecimalToBinary(String hexadecimal){
        String hold = Strings.hexadecimalToDecimalString(hexadecimal);
        return Strings.decimalToBinary(hold);
    }
    
    // hexadecimal to octal
    // static method
    public static String hexadecimalToOctal(String hexadecimal){
        String hold = Strings.hexadecimalToDecimalString(hexadecimal);
        return Strings.decimalToOctal(hold);
    }
        
    // Octal to decimal
    // static method
    public static int octalToDecimal(String octal){
        return Integer.parseInt(octal,8);
    }
    
    public static String octalToDecimalString(String octal){
        return String.valueOf(Integer.parseInt(octal,8));
    }
    
     // Octal to hexadecimal
    // static method
    public static String octalToHexadecimal(String octal){
        String hold = Strings.octalToDecimalString(octal);
        return Strings.decimalToHexadecimal(hold);
    }
    
    // Octal to binary
    // static method
    public static String octalToBinary(String octal){
        String hold = Strings.octalToDecimalString(octal);
        return Strings.decimalToBinary(hold);
    }
    
      
     // HTML decimal code to HTML hexadecimal code
     public static String htmlDecimalCodeToHtmlHexadecimalCode(String decCode){
         int dec = Strings.htmlDecimalCodeToAsciiDecimalCode(decCode);
         String hdecs = Strings.decimalToHexadecimal(decCode);
         return Strings.asciiHexadecimalCodeToHtmlHexadecimalNumberCode(hdecs);
     }
      
     // HTML decimal code toASCII decimal code
     public static int htmlDecimalCodeToAsciiDecimalCode(String dcode){
         boolean test0 = true;
         String retns = null;
         int retni = -1;
         int pos0 = dcode.indexOf("&#");
         int pos1 = dcode.indexOf(";");
         if(pos0<0){
             test0=false;
         }
         else{
             if(pos1<0){
                 test0=false;
             }
             else{
                 retns = dcode.substring(pos0+2, pos1);
                 retni = Integer.parseInt(retns);
             }
         }
         if(!test0)  throw new IllegalArgumentException("Argument " + dcode + " does not appear to be a valid html decimal number code!");
         return retni;
     }
     
       // HTML decimal number code to ASCII hexadecimal code
     public static String htmlDecimalCodeToAsciiHexadecimalCode(String decCode){
         int dec = Strings.htmlDecimalCodeToAsciiDecimalCode(decCode);
         return Strings.decimalToHexadecimal(dec);
     }
      
        
       // HTML decimal number code toHTML name code
     public static String htmlDecimalCodeToHtmlNameCode(String numcode){
         Strings ssss = new Strings();
         String retn = null;
         boolean test0 = true;
         for(int i=0; i<ssss.nSymbolsNA; i++){
             if(numcode.equals(ssss.htmlNumbersNA[i])){
                retn = ssss.htmlNamesNA[i];
                test0 = false;
             }
             if(!test0)break;
         }
         if(test0)throw new IllegalArgumentException("Argument " + numcode + " does not have a matching name code in this program's database");
         return retn;
     }
     
     // HTML name code to html decimal number code
     public static String htmlNameCodeToHtmlDecimalCode(String namecode){
         Strings ssss = new Strings();
         String retn = null;
         boolean test0 = true;
         for(int i=0; i<ssss.nSymbolsNA; i++){
             if(namecode.equals(ssss.htmlNamesNA[i])){
                retn = ssss.htmlNumbersNA[i];
                test0 = false;
             }
             if(!test0)break;
         }
         if(test0)throw new IllegalArgumentException("Argument " + namecode + " does not have a matching name code in this program's database");
         return retn;
     }
     
     // HTML name code to html hexadecimal number code
     public static String htmlNameCodeToHtmlHexadecimalCode(String namecode){
         Strings ssss = new Strings();
         String retn = null;
         boolean test0 = true;
         for(int i=0; i<ssss.nSymbolsNA; i++){
             if(namecode.equals(ssss.htmlNamesNA[i])){
                retn = ssss.htmlHexadecimalsNA[i];
                test0 = false;
             }
             if(!test0)break;
         }
         if(test0)throw new IllegalArgumentException("Argument " + namecode + " does not have a matching name code in this program's database");
         return retn;
     }
     
    // HTML name code to ascii hexadecimal  code
     public static int htmlNameCodeToAsciDecimalCode(String namecode){
         Strings ssss = new Strings();
         int retn = -1;
         boolean test0 = true;
         for(int i=0; i<ssss.nSymbolsNA; i++){
             if(namecode.equals(ssss.htmlNamesNA[i])){
                retn = ssss.decNumbersNA[i];
                test0 = false;
             }
             if(!test0)break;
         }
         if(test0)throw new IllegalArgumentException("Argument " + namecode + " does not have a matching name code in this program's database");
         return retn;
     }
     
     
     // HTML name code to html hexadecimal number code
     public static String htmlNameCodeToAsciiHexadecimalCode(String namecode){
         Strings ssss = new Strings();
         String retn = null;
         boolean test0 = true;
         for(int i=0; i<ssss.nSymbolsNA; i++){
             if(namecode.equals(ssss.htmlNamesNA[i])){
                retn = ssss.hexadecsNA[i];
                test0 = false;
             }
             if(!test0)break;
         }
         if(test0)throw new IllegalArgumentException("Argument " + namecode + " does not have a matching name code in this program's database");
         return retn;
     }
     
    // HTML hexadecimal number code toHTML name code
     public static String htmlHexadecimalCodeToHtmlNameCode(String numcode){
         Strings ssss = new Strings();
         String retn = null;
         boolean test0 = true;
         for(int i=0; i<ssss.nSymbolsNA; i++){
             if(numcode.equals(ssss.htmlHexadecimalsNA[i])){
                retn = ssss.htmlNamesNA[i];
                test0 = false;
             }
             if(!test0)break;
         }
         if(test0)throw new IllegalArgumentException("Argument " + numcode + " does not have a matching name code in this program's database");
         return retn;
     }
     
     
      // HTML hexadecimal code to HTML decimal code
     public static String htmlhexaecimalCodeToHtmlDecimalCode(String hexCode){
         int dec = Strings.htmlHexadecimalCodeToAsciiDecimalCode(hexCode);
         String hdecs = Strings.decimalToHexadecimal(dec);
         return Strings.asciiDecimalCodeToHtmlDecimalNumberCode(hdecs);
     }
     
     // HTML hexadecimal code toASCII decimald code
     public static int htmlHexadecimalCodeToAsciiDecimalCode(String hcode){
         boolean test0 = true;
         String retns = null;
         int retni = -1;
         int pos0 = hcode.indexOf("&#x");
         int pos1 = hcode.indexOf(";");
         if(pos0<0){
             test0=false;
         }
         else{
             if(pos1<0){
                 test0=false;
             }
             else{
                 retns = hcode.substring(pos0+3, pos1);
                 retni = Strings.hexadecimalToDecimal(hcode);
             }
         }
         if(!test0)  throw new IllegalArgumentException("Argument " + hcode + " does not appear to be a valid html hexadecimal number code!");
         return retni;
     }
    
      // HTML hexadecimal code toASCII hexadecimal code
     public static String htmlHexadecimalCodeToAsciiHexadecimalCode(String hcode){
         boolean test0 = true;
         String retns = null;
         int retni = -1;
         int pos0 = hcode.indexOf("&#x");
         int pos1 = hcode.indexOf(";");
         if(pos0<0){
             test0=false;
         }
         else{
             if(pos1<0){
                 test0=false;
             }
             else{
                 retns = hcode.substring(pos0+3, pos1);
             }
         }
         if(!test0)  throw new IllegalArgumentException("Argument " + hcode + " does not appear to be a valid html hexadecimal number code!");
         return retns;
     }
    

    // ASCII decimal to HTML decimal number code
     public static String asciiDecimalCodeToHtmlDecimalNumberCode(int dec){
        String decs = String.valueOf(dec);
        return Strings.asciiDecimalCodeToHtmlDecimalNumberCode(decs);
     }
         
     public static String asciiDecimalCodeToHtmlDecimalNumberCode(String decs){
         String retn = "&#" + decs + ";";
         return retn;
     }
     
      // ASCII decimal to HTML hexadecimal number code
     public static String asciiDecimalCodeToHtmlHexadecimalNumberCode(int dec){
        String decs = String.valueOf(dec);
        return Strings.asciiDecimalCodeToHtmlHexadecimalNumberCode(decs);
     }
     
     public static String asciiDecimalCodeToHtmlHexadecimalNumberCode(String decs){
         String hex = Strings.decimalToHexadecimal(decs);
         String retn = "&#x" + hex + ";";
         return retn;
     }
     
      // ASCII decimal number code to HTML name code
     public static String asciidecimalCodeToHtmlNameCode(int dec){
         Strings ssss = new Strings();
         String retn = null;
         boolean test0 = true;
         for(int i=0; i<ssss.nSymbolsNA; i++){
             if(dec==ssss.decNumbersNA[i]){
                retn = ssss.htmlNamesNA[i];
                test0 = false;
            }
            if(!test0)break;
         }
         if(test0)throw new IllegalArgumentException("Argument " + dec + " does not have a matching name code in this program's database");
         return retn;
     }
     
     public static String asciidecimalCodeToHtmlNameCode(String dec){
         int deci = Integer.parseInt(dec);
         return Strings.asciidecimalCodeToHtmlNameCode(deci);
     }
          
      // ASCII hexadecimal to HTML hexadecimal number code
     public static String asciiHexadecimalCodeToHtmlHexadecimalNumberCode(String hdecs){
         String retn = "&#x" + hdecs + ";";
         return retn;
     }
     
     
      // ASCII hexadecimal to HTML decimal number code
     public static String asciiHexadecimalCodeToHtmlDecimalNumberCode(String hdecs){
         String dec = Strings.decimalToHexadecimal(hdecs);
         String retn = "&#" + dec + ";";
         return retn;
     }
     
     // ASCII hexadecimal number code to HTML name code
     public static String asciiHexadecimalCodeToHtmlNameCode(String hex){
         Strings ssss = new Strings();
         String retn = null;
         boolean test0 = true;
         for(int i=0; i<ssss.nSymbolsNA; i++){
             if(hex.equals(ssss.hexadecsNA[i])){
                retn = ssss.htmlNamesNA[i];
                test0 = false;
             }
             if(!test0)break;
         }
         if(test0)throw new IllegalArgumentException("Argument " + hex + " does not have a matching name code in this program's database");
         return retn;
     }
     
     
     // Get operating system
     public static String getOperatingSystem() {
        String os = System.getProperty("os.name");
        return os;
    }
     
     // Get file encoding
     public static String getFileEncoding() {
        String fe = System.getProperty("file.encoding");
        return fe;
    }
     
      // Get Sun jnr encoding
     public static String getSunJnrEncoding() {
        String sje = System.getProperty("sun.jnr.encoding");
        return sje;
    }
     
    // Display encoding and operating system name
     public static void displayEncoding(){
        String os = System.getProperty("os.name");
        String fe = System.getProperty("file.encoding");
        String sje = System.getProperty("sun.jnu.encoding");
        String message = "Operating system:  " + os + "\n\nFile encoding:  " + fe + "\n\nSun jnr encoding:  "  + sje;
        DialogueBox.show("Encoding", message, "");
    }
     
     
}